# import of libraries
import numpy as np
import sys
import cProfile, pstats, io

np.set_printoptions(threshold=sys.maxsize)


def Pendulum_Force (iteration_step,x_list,Angle_Pendulum,Pendulum_hinge_top_location,Stiffness_Pendulum):
    
    #This function calculates the elongation of the pendulum and uses it to determine the axial pendulum force.
    
    
    #Assumption the Angle_Penulum, y, z, and phi is constant within the time step
    #This method does not make use of small angle approximations. So no assumptions here.
    
    #Create empty forcing vector
    F = np.zeros(3)   
    
    #Pendulum location and geometry
    y_loc_p = abs(Pendulum_hinge_top_location[0])  #Fixed geometry value for the hinge location of the pendulum with respect to the COG
    z_loc_p = abs(Pendulum_hinge_top_location[1]) #Fixed geometry value for the hinge location of the pendulum with respect to the COG
    l_yz_loc_p = np.sqrt(y_loc_p**2+z_loc_p**2)  #Fixed length. No need to update every time step
    theta = np.arctan(z_loc_p/y_loc_p)    #Fixed angle. No need to update every time step
    
    #Find the displacement and the pendulum angle of the element of this time step
    y = x_list[iteration_step,0] #Sway displacement of the COG at the beginning of the time step
    z = x_list[iteration_step,1] #Heave displacement of the COG at the beginning of the time step
    phi = x_list[iteration_step,2] #Roll displacement of the COG at the beginning of the time step
    Angle_Pendulum = Angle_Pendulum[0]    #This angle is updated every time step. Assumption for small differences within the time step
    
    #Find the variable geometry dependent on the roll displacement
    d_r = np.sin(phi/2) * l_yz_loc_p
    alpha = 0.5 * (np.pi + phi) - theta #With the use of two statements: 1. all angles summed in a triangle are equal to pi (for the triangle 1), and 2. the sum of all angles within a strait line sum up to pi.
    y_r = np.cos(alpha) * 2 * d_r #2 times d_r
    z_r = np.sin(alpha) * 2 * d_r #2 times d_r
    
    d_l_y = np.cos(Angle_Pendulum) * (y + y_r) # cos(angle) * (y_contribution_sway + y_contribution_roll)
    d_l_z = np.sin(Angle_Pendulum) * (z + z_r) # sin(angle) * (z_contribution_heave + z_contribution_roll)
    d_l_tot = d_l_y + d_l_z
    
    F_axial = d_l_tot * Stiffness_Pendulum[0]
 
    F[0] = -2 * F_axial * np.cos(Angle_Pendulum)  #2 Pendulums
    F[1] = -2 * F_axial * np.sin(Angle_Pendulum) #2 Pendulums
    F[2] = -2 * (F_axial * np.cos(Angle_Pendulum) * np.sin(theta - phi) * l_yz_loc_p + F_axial * np.sin(Angle_Pendulum) * np.cos(theta - phi) * l_yz_loc_p) #2 Pendulums

    
    return(F,F_axial)


def Constant(segments, steps, Three_Dimensional_Calculation):
    """"
    # Function:
# This function makes a constant horizontal force, applied on the height of the COG.
# Mainly to test numerical scheme vs analytical scheme.

# Input:
# segments                          		= Number of segments in the bridge                      	(1)
# steps                             		= Number of steps that the dynamic model will perform   	(1)
# Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)

# Output:
# F                 						= Force list for every step                                 ((steps) x N)
"""

    # Size of matrix
    if Three_Dimensional_Calculation == True:
        N_matrix = 6 * segments
    else:
        N_matrix = 3

    # Force list for every step
    F = np.zeros((steps, N_matrix))

    # Fill force list
    if Three_Dimensional_Calculation == True:
        for i in range(segments):
            F[:, 1 + 6 * i] = 5000000 * np.ones(steps)
    else:
        F[:, 0] = 5000000 * np.ones(steps)

    return (F)


def FloaterDrag(iteration_step, Total_time, x_list, xdot_list, segments, steps, characteristics_small_floater,
                characteristics_large_floater,
                M_segment, C_drag_pointed_rectangle, C_drag_bottom_floater, z_location_bottom_floater, Tidal_level,
                flow_velocity, Wave_hlp, rho, depth, Current_Force, Wave_Force,
                Three_Dimensional_Calculation, Approaching_Wave_Field):
    """"
    # Function:
# This function makes an estimation of the drag force and moment, assuming constant flow velocity over height and
# using the DNV square rod equation, using a height of double the draught and a density half that of water.

# Input:
# iteration_step    						= Iteration step number                                 	(1)
# Total_time								= Total time that is simulated by the model					(1)
# segments                          		= Number of segments in the bridge                      	(1)
# x_list            						= Storage of location as function of time               	((steps + 1) x N)
# xdot_list         						= Storage of velocity as function of time               	((steps + 1) x N)
# steps                             		= Number of steps that the dynamic model will perform   	(1)
# characteristics_small_floater     		= Number of, width and length of small floaters         	(3)
# characteristics_large_floater     		= Number of, width and length of large floaters         	(3)
# M_segment                         		= Mass of a single segment                              	(1)
# C_drag_pointed_rectangle          		= Drag coefficient of a pointed rectangle               	(1)
# C_drag_bottom_floater                     = Drag coefficient of vertical flow against floaters        (1)
# Tidal_level								= Tidal level to be evaluated								(1)
# flow_velocity                     		= Flow velocity of the fluid                            	(1)
# Wave_hlp									= Regular wave height, length and period					(3)
# depth										= Local water depth											(1)
# rho                               		= Density of the fluid                                  	(1)
# z_location_bottom_floater         		= Z-coordinate of the bottom side of the floaters       	(1)
# Current_Force								= Option to implement force by current on floaters			(T/F)
# Wave_Force								= Option to implement force by waves on floaters			(T/F)
# Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)

# Output:
# F                 						= Force list for every step                                 ((steps) x N)
"""

    # seperate data from arrays which were imported
    N_small_floater, W_small_floater, L_small_floater = characteristics_small_floater
    N_large_floater, W_large_floater, L_large_floater = characteristics_large_floater

    # Horizontal area of floaters, including the tips
    A_small_floater = W_small_floater * L_small_floater + 2 * 1 / 2. * W_small_floater * W_small_floater  # m2
    A_large_floater = W_large_floater * L_large_floater + 2 * 1 / 2. * W_large_floater * 0.5 * W_large_floater  # m2

    A_total = (A_small_floater * N_small_floater + A_large_floater * N_large_floater)

    # draught of structure, M / A. and relative velocity
    constant_draught = M_segment / (A_total * rho)

    if Current_Force == False:
        flow_velocity = 0

    draught = constant_draught - x_list[iteration_step, 1]
    relative_horizontal_velocity = flow_velocity - xdot_list[iteration_step, 0]
    relative_vertical_velocity = - xdot_list[iteration_step, 1]
    relative_rotational_velocity = - xdot_list[iteration_step, 2]

    if Wave_Force == True:
        time = np.linspace(0, Total_time, steps)[iteration_step]
        Wave_height, Wave_length, Wave_period = Wave_hlp

        # Fix wave direction, as negative waves are entered for negative direction
        if Wave_height < 0:
            Wave_height = - Wave_height
            Wave_direction = -1
        else:
            Wave_direction = 1

        # Wave parameters
        Wave_k = 2 * np.pi / Wave_length
        Wave_w = 2 * np.pi / Wave_period
        a = Wave_height / 2.

        # Depth fix, include additional water level due to tides
        depth = depth + Tidal_level

        # correction for horizontal position of the bridge as this will delay wave impact

        y_bridge = x_list[iteration_step, 0]

        # Wave height as function of y location, empty array
        eta_small = np.zeros(2)
        eta_large = np.zeros(2)

        u_y_small = np.zeros(2)
        u_y_large = np.zeros(2)

        # Calculate force on every segment, skip first and last if 2d assessment
        
        y_space_small = (L_small_floater / 2. + 1 / 2. * W_small_floater) * np.array([-1, 1])
        y_space_large = (L_large_floater / 2. + 1 / 4. * W_large_floater) * np.array([-1, 1])

        for j in range(2):
            eta_small[j] = a * np.cos(
                Wave_w * time + flow_velocity * time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (
                        y_space_small[j] + y_bridge)) + Wave_k * a ** 2 * np.cosh(Wave_k * depth) / (
                                      4 * (np.sinh(Wave_k * depth)) ** 3) * (
                                      2 + np.cosh(2 * Wave_k * depth)) * np.cos(
                2 * (Wave_w * time + flow_velocity * time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (
                            y_space_small[j] + y_bridge)))

            eta_large[j] = a * np.cos(
                Wave_w * time + flow_velocity * time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (
                        y_space_large[j] + y_bridge)) + Wave_k * a ** 2 * np.cosh(Wave_k * depth) / (
                                      4 * (np.sinh(Wave_k * depth)) ** 3) * (
                                      2 + np.cosh(2 * Wave_k * depth)) * np.cos(
                2 * (Wave_w * time + flow_velocity * time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (
                            y_space_large[j] + y_bridge)))

            
            u_y_small_zcoordinate = np.zeros(14)
            u_y_large_zcoordinate = np.zeros(14)

            gridsize_small = (eta_small[j] + draught) / 14
            z_small = np.linspace(-draught + gridsize_small / 2., eta_small[j] - gridsize_small / 2., 14)

            gridsize_large = (eta_small[ j] + draught) / 14
            z_large = np.linspace(-draught + gridsize_large / 2., eta_large[j] - gridsize_large / 2., 14)

            #Average over the height
            for k in range(14):
                u_y_small_zcoordinate[k] = Wave_w * a * np.cosh(Wave_k * (z_small[k] + depth)) / np.sinh(
                    Wave_k * depth) * np.cos(
                    Wave_w * time + flow_velocity * time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (
                                y_space_small[
                                    j] + y_bridge)) + 3 / 4. * Wave_w * a ** 2 * Wave_k * np.cosh(
                    2 * Wave_k * (z_small[k] + depth)) / (
                                                    (np.cosh(Wave_k * depth)) ** 3 * np.sinh(
                                                Wave_k * depth)) * np.cos(
                    2 * (
                                Wave_w * time + flow_velocity * time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (
                                    y_space_small[j] + y_bridge))) \
                                            + relative_horizontal_velocity - relative_rotational_velocity * (z_small[k] + z_location_bottom_floater + draught)

                u_y_large_zcoordinate[k] = Wave_w * a * np.cosh(Wave_k * (z_large[k] + depth)) / np.sinh(
                    Wave_k * depth) * np.cos(
                    Wave_w * time + flow_velocity * time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (
                                y_space_large[
                                    j] + y_bridge)) + 3 / 4. * Wave_w * a ** 2 * Wave_k * np.cosh(
                    2 * Wave_k * (z_large[k] + depth)) / (
                                                    (np.cosh(Wave_k * depth)) ** 3 * np.sinh(
                                                Wave_k * depth)) * np.cos(
                    2 * (
                                Wave_w * time + flow_velocity * time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (
                                    y_space_large[j] + y_bridge))) \
                                            + relative_horizontal_velocity - relative_rotational_velocity * (z_large[k] + z_location_bottom_floater + draught)

                

            u_y_small[j] = np.sum(u_y_small_zcoordinate) / 14.
            u_y_large[j] = np.sum(u_y_large_zcoordinate) / 14.

            if gridsize_small < 0:
                u_y_small[j] = 0
            if gridsize_large < 0:
                u_y_large[j] = 0


        Height_representative_small = np.zeros(2)
        Height_representative_large = np.zeros(2)

        Height_representative_small[0] = 2 * (draught + eta_small[0])
        Height_representative_small[1] = 2 * (draught + eta_small[1])
        Height_representative_large[0] = 2 * (draught + eta_large[0])
        Height_representative_large[1] = 2 * (draught + eta_large[1])

        A_hitbox_small = W_small_floater * N_small_floater * Height_representative_small
        A_hitbox_large = W_large_floater * N_large_floater * Height_representative_large

        Drag_force = 0
        Drag_force += 1 / 2. * C_drag_pointed_rectangle * (rho + 0) / 2. * u_y_small[0] * abs(u_y_small[ 0]) * A_hitbox_small[ 0] / 2
        Drag_force += 1 / 2. * C_drag_pointed_rectangle * (rho + 0) / 2. * u_y_small[1] * abs(u_y_small[ 1]) * A_hitbox_small[ 1] / 2
        Drag_force += 1 / 2. * C_drag_pointed_rectangle * (rho + 0) / 2. * u_y_large[0] * abs(u_y_large[ 0]) * A_hitbox_large[ 0] / 2
        Drag_force += 1 / 2. * C_drag_pointed_rectangle * (rho + 0) / 2. * u_y_large[1] * abs(u_y_large[1]) * A_hitbox_large[ 1] / 2

        
        
        Drag_moment = 0
        Drag_moment += 1 / 2. * C_drag_pointed_rectangle * (rho + 0) / 2. * u_y_small[ 0] * abs(
            u_y_small[0]) * A_hitbox_small[ 0] / 2. * (
                            -(z_location_bottom_floater + 1 / 4. * Height_representative_small[ 0]))
        Drag_moment += 1 / 2. * C_drag_pointed_rectangle * (rho + 0) / 2. * u_y_small[1] * abs(
            u_y_small[1]) * A_hitbox_small[ 1] / 2. * (
                            -(z_location_bottom_floater + 1 / 4. * Height_representative_small[ 1]))

        Drag_moment += 1 / 2. * C_drag_pointed_rectangle * (rho + 0) / 2. * u_y_large[ 0] * abs(
            u_y_large[0]) * A_hitbox_large[ 0] / 2 * (
                            -(z_location_bottom_floater + 1 / 4. * Height_representative_small[ 0]))
        Drag_moment += 1 / 2. * C_drag_pointed_rectangle * (rho + 0) / 2. * u_y_large[1] * abs(
            u_y_large[ 1]) * A_hitbox_large[1] / 2 * (
                            -(z_location_bottom_floater + 1 / 4. * Height_representative_small[1]))

                
        
                
        
        Length_small = L_small_floater + 2 * W_small_floater / 2
        Length_large = L_large_floater + W_large_floater / 2

        y_loc_coordinate_small = np.linspace(-Length_small / 2 + Length_small / 28,
                                              Length_small / 2 - Length_small / 28, 14)
        y_loc_coordinate_large = np.linspace(-Length_large / 2 + Length_large / 28,
                                              Length_large / 2 - Length_large / 28,
                                              14)

        Vertical_Flow_Force = 0
        Vertical_Flow_Moment = 0

        u_z_small = np.zeros(14)
        u_z_large = np.zeros(14)

        for j in range(14):
            z_bottom_small = -draught + x_list[iteration_step, 2] * y_loc_coordinate_small[j]
            z_bottom_large = -draught + x_list[iteration_step, 2] * y_loc_coordinate_large[j]

            if z_bottom_small < 0:
                u_z_small[j] = Wave_w * a * np.sinh(Wave_k * (-draught + depth)) / np.sinh(
                    Wave_k * depth) * np.sin(
                    Wave_w * time + flow_velocity * time / Wave_length * 2 * np.pi - Wave_direction * Wave_k *
                    (y_loc_coordinate_small[
                          j] + y_bridge)) + 3 / 4. * Wave_w * a ** 2 * Wave_k * np.sinh(
                    2 * Wave_k * (-draught + depth)) / (
                                        (np.cosh(Wave_k * depth)) ** 3 * np.sinh(Wave_k * depth)) * np.sin(
                    2 * (Wave_w * time + flow_velocity * time / Wave_length * 2 * np.pi - Wave_direction * Wave_k *
                          (y_loc_coordinate_small[j] + y_bridge))) + relative_vertical_velocity + \
                                relative_rotational_velocity * y_loc_coordinate_small[j]


            if z_bottom_large < 0:
                u_z_large[j] = Wave_w * a * np.sinh(Wave_k * (-draught + depth)) / np.sinh(
                    Wave_k * depth) * np.sin(
                    Wave_w * time + flow_velocity * time / Wave_length * 2 * np.pi - Wave_direction * Wave_k *
                    (y_loc_coordinate_large[
                          j] + y_bridge)) + 3 / 4. * Wave_w * a ** 2 * Wave_k * np.sinh(
                    2 * Wave_k * (-draught + depth)) / (
                                        (np.cosh(Wave_k * depth)) ** 3 * np.sinh(
                                    Wave_k * depth)) * np.sin(
                    2 * (Wave_w * time + flow_velocity * time / Wave_length * 2 * np.pi - Wave_direction * Wave_k *
                          (y_loc_coordinate_large[j] + y_bridge))) + relative_vertical_velocity + \
                                relative_rotational_velocity * y_loc_coordinate_large[j]

        

        for j in range(14):
            Vertical_Flow_Force += 1 / 2. * C_drag_bottom_floater * rho * (
                        W_small_floater * Length_small / 14) * N_small_floater * abs(u_z_small[j]) * u_z_small[j] \
                                            + 1 / 2. * C_drag_bottom_floater * rho * (
                                                        W_large_floater * Length_large / 14) * N_large_floater * abs(
                u_z_large[j]) * u_z_large[j]
            Vertical_Flow_Moment += 1 / 2. * C_drag_bottom_floater * rho * (
                        W_small_floater * Length_small / 14) * N_small_floater * abs(u_z_small[j]) * u_z_small[j] * \
                                              y_loc_coordinate_small[j] \
                                              + 1 / 2. * C_drag_bottom_floater * rho * (
                                                          W_large_floater * Length_large / 14) * N_large_floater * abs(
                u_z_large[j]) * u_z_large[j] * y_loc_coordinate_large[j]
                                                        

    else:
        # representitive height and width of drag hitbox, two times draught as structure can be mirrored on water level
        Height_representative = 2 * draught
        W_total_floaters = W_small_floater * N_small_floater + W_large_floater * N_large_floater

        # Area of hitbox, width x height
        A_hitbox = W_total_floaters * Height_representative

        if A_hitbox < 0:
            A_hitbox = 0

        # Force and moment of drag force, F = 1/2. * rho * C * v^2 * A, rho devided by two to allow mirror on water level
        Drag_force = 1 / 2. * C_drag_pointed_rectangle * (rho + 0) / 2. * relative_horizontal_velocity * abs(relative_horizontal_velocity) * A_hitbox

        Drag_moment = -(z_location_bottom_floater + 1 / 2. * draught) * Drag_force

        Vertical_Flow_Force = 0
        Vertical_Flow_Moment = 0

        Length_small = L_small_floater + 2 * W_small_floater / 2
        Length_large = L_large_floater + W_large_floater / 2

        y_loc_coordinate_small = np.linspace(-Length_small / 2 + Length_small / 28,
                                              Length_small / 2 - Length_small / 28, 14)
        y_loc_coordinate_large = np.linspace(-Length_large / 2 + Length_large / 28,
                                              Length_large / 2 - Length_large / 28,
                                              14)



        u_z_small = np.zeros(14)
        u_z_large = np.zeros(14)

        for j in range(14):
            u_z_small[j] = relative_vertical_velocity + relative_rotational_velocity * \
                            y_loc_coordinate_small[j]
            u_z_large[j] = relative_vertical_velocity + relative_rotational_velocity * \
                            y_loc_coordinate_large[j]

        for j in range(14):
            z_bottom_small = -draught - x_list[iteration_step, 2] * y_loc_coordinate_small[j]
            z_bottom_large = -draught - x_list[iteration_step, 2] * y_loc_coordinate_large[j]

            if z_bottom_small < 0:
                Vertical_Flow_Force += 1 / 2. * C_drag_bottom_floater * rho * (
                            W_small_floater * Length_small / 14) * N_small_floater * abs(u_z_small[j]) * u_z_small[
                                                    j] \
                                                + 1 / 2. * C_drag_bottom_floater * rho * (
                                                            W_large_floater * Length_large / 14) * N_large_floater * \
                                                abs(u_z_large[j]) * u_z_large[j]
            if z_bottom_large < 0:
                Vertical_Flow_Moment += 1 / 2. * C_drag_bottom_floater * rho * (
                            W_small_floater * Length_small / 14) * N_small_floater * abs(u_z_small[j]) * u_z_small[
                                                      j] * y_loc_coordinate_small[j] \
                                                  + 1 / 2. * C_drag_bottom_floater * rho * (
                                                              W_large_floater * Length_large / 14) * N_large_floater * \
                                                  abs(u_z_large[j]) * u_z_large[j] * y_loc_coordinate_large[j]


    N_matrix = 3
    # Empty list of forces for every step
    F = np.zeros((steps, N_matrix))

    # Fill list for every step
    F[iteration_step, 0] = Drag_force
    F[iteration_step, 1] = Vertical_Flow_Force
    F[iteration_step, 2] = Drag_moment + Vertical_Flow_Moment

    return (F)


def TurbineDrag(iteration_step, Total_time, x_list, xdot_list, steps, flow_velocity, Wave_hlp, rho, depth, Current_Force,
                Wave_Force, Turbine_dimensions_HWL ,Turbine_char, draught,Tidal_level, z_location_bottom_floater, z_location_turbines):
        


    """"
    # Function:
# This function makes an estimation of the drag force and moment, assuming constant flow velocity over height and
# using the DNV square rod equation, using a height of double the draught and a density half that of water.

# Input:
# iteration_step    						= Iteration step number                                 	(1)
# Total_time								= Total time that is simulated by the model					(1)
# segments                          		= Number of segments in the bridge                      	(1)
# x_list            						= Storage of location as function of time               	((steps + 1) x N)
# xdot_list         						= Storage of velocity as function of time               	((steps + 1) x N)
# steps                             		= Number of steps that the dynamic model will perform   	(1)
# characteristics_small_floater     		= Number of, width and length of small floaters         	(3)
# characteristics_large_floater     		= Number of, width and length of large floaters         	(3)
# M_segment                         		= Mass of a single segment                              	(1)
# C_drag_pointed_rectangle          		= Drag coefficient of a pointed rectangle               	(1)
# C_drag_bottom_floater                     = Drag coefficient of vertical flow against floaters        (1)
# Tidal_level								= Tidal level to be evaluated								(1)
# flow_velocity                     		= Flow velocity of the fluid                            	(1)
# Wave_hlp									= Regular wave height, length and period					(3)
# depth										= Local water depth											(1)
# rho                               		= Density of the fluid                                  	(1)
# z_location_bottom_floater         		= Z-coordinate of the bottom side of the floaters       	(1)
# Current_Force								= Option to implement force by current on floaters			(T/F)
# Wave_Force								= Option to implement force by waves on floaters			(T/F)
# Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)

# Output:
# F                 						= Force list for every step                                 ((steps) x N)
"""

    

    # seperate data from arrays which were imported
    FFTurbine_H, FFTurbine_W, FFTurbine_L, Turbine_depth, Tur_grid_size, Turbine_Volume, N_turbines, FFTur_evalf_loc = Turbine_dimensions_HWL
    Diameter_turbine_impeller, Induction_factor, C_added_tur_hori, C_added_tur_vert, C_drag_turbine = Turbine_char

    if Current_Force == False:
        flow_velocity = 0

    draught = draught - x_list[iteration_step, 1]
    A_hitbox = N_turbines *  (FFTurbine_H * FFTurbine_W - 0.25 * np.pi * Diameter_turbine_impeller**2)
    A_bottom_turbine = N_turbines * FFTurbine_W * FFTurbine_L
    
    if Wave_Force == True:
        time = np.linspace(0, Total_time, steps)[iteration_step]
        Wave_height, Wave_length, Wave_period = Wave_hlp
        # Fix wave direction, as negative waves are entered for negative direction
        if Wave_height < 0:
            Wave_height = - Wave_height
            Wave_direction = -1
        else:
            Wave_direction = 1
        # Wave parameters
        Wave_k = 2 * np.pi / Wave_length
        Wave_w = 2 * np.pi / Wave_period
        a = Wave_height / 2.
        # Depth fix, include additional water level due to tides
        depth = depth + Tidal_level
        # correction for horizontal position of the bridge as this will delay wave impact
        y_bridge = x_list[iteration_step, 0]


##### HORIZONTAL CONTRIBUTION #####
        # eta = np.zeros(3)
        u_y = np.zeros(3)
        # Calculate force on every segment, skip first and last if 2d assessment
        y_space = FFTurbine_L / 2. * np.array([-1, 0, 1])
        for j in range(3):
            # eta[j] = a * np.cos(Wave_w * time + flow_velocity * time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (
            #          y_space[j] + y_bridge)) + Wave_k * a ** 2 * np.cosh(Wave_k * depth) / (4 * (np.sinh(Wave_k * depth)) ** 3) * (
            #          2 + np.cosh(2 * Wave_k * depth)) * np.cos(2 * (Wave_w * time + flow_velocity * time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (
            #          y_space[j] + y_bridge)))

            u_y_zcoordinate = np.zeros(14)
            gridsize = FFTurbine_H / 14.
            z = np.linspace(-Turbine_depth - FFTurbine_H + gridsize / 2., -Turbine_depth - gridsize / 2., 14)

            #Average over the height
            for k in range(14):
                u_y_zcoordinate[k] = Wave_w * a * np.cosh(Wave_k * (z[k] + depth)) / np.sinh(Wave_k * depth) * np.cos(
                    Wave_w * time + flow_velocity * time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (
                                y_space[j] + y_bridge)) + 3 / 4. * Wave_w * a ** 2 * Wave_k * np.cosh(
                    2 * Wave_k * (z[k] + depth)) / ((np.cosh(Wave_k * depth)) ** 3 * np.sinh(Wave_k * depth)) * np.cos(
                    2 * (Wave_w * time + flow_velocity * time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (
                     y_space[j] + y_bridge))) + flow_velocity - xdot_list[iteration_step, 0] + xdot_list[iteration_step, 2] * (z[k] + FFTurbine_H / 2. + z_location_turbines + Turbine_depth)

            u_y[j] = np.sum(u_y_zcoordinate) / 14.
            if gridsize < 0:
                u_y[j] = 0

        Drag_force = 0
        Drag_force += 1 / 2. * C_drag_turbine * rho / 2. * u_y[0] * abs(u_y[ 0]) * A_hitbox
        Drag_force += 1 / 2. * C_drag_turbine * rho / 2. * u_y[2] * abs(u_y[ 1]) * A_hitbox
        Turbine_Force = 0.5 * rho * u_y[1]**3 * 0.25*np.pi*Diameter_turbine_impeller * 4 * Induction_factor * (1-Induction_factor)**2
        
        Drag_moment = 0
        Drag_moment += 1 / 2. * C_drag_turbine * rho / 2. * u_y[0] * abs(u_y[0]) * A_hitbox * (-(z_location_turbines))
        Drag_moment += 1 / 2. * C_drag_turbine * rho / 2. * u_y[2] * abs(u_y[1]) * A_hitbox * (-(z_location_turbines))
        Turbine_Moment = - Turbine_Force * z_location_turbines



        
##### VERTICAL CONTRIBUTION #######3
        Vertical_Flow_Force = 0
        Vertical_Flow_Moment = 0
        u_z = np.zeros(14)
        y_loc_coordinate = np.linspace(-FFTurbine_L / 2 + FFTurbine_L / 28, FFTurbine_L / 2 - FFTurbine_L / 28, 14)

        for j in range(14):
            z_bottom = - Turbine_depth - FFTurbine_H + x_list[iteration_step, 1] + x_list[iteration_step, 2] * y_loc_coordinate[j]
            if z_bottom < 0:
                u_z[j] = Wave_w * a * np.sinh(Wave_k * (z_bottom + depth)) / np.sinh(Wave_k * depth) * np.sin(
                    Wave_w * time + flow_velocity * time / Wave_length * 2 * np.pi - Wave_direction * Wave_k *
                    (y_loc_coordinate[j] + y_bridge)) + 3 / 4. * Wave_w * a ** 2 * Wave_k * np.sinh(
                    2 * Wave_k * (z_bottom + depth)) / ((np.cosh(Wave_k * depth)) ** 3 * np.sinh(Wave_k * depth)) * np.sin(
                    2 * (Wave_w * time + flow_velocity * time / Wave_length * 2 * np.pi - Wave_direction * Wave_k *
                    (y_loc_coordinate[j] + y_bridge))) - xdot_list[iteration_step, 1] - xdot_list[iteration_step, 2] * y_loc_coordinate[j]

            Vertical_Flow_Force += 1 / 2. * C_drag_turbine * rho * A_bottom_turbine / 14. * abs(u_z[j]) * u_z[j]
            Vertical_Flow_Moment += 1 / 2. * C_drag_turbine * rho * A_bottom_turbine / 14. * abs(u_z[j]) * u_z[j] * y_loc_coordinate[j]

#### No wave forcing #####                                              
    else:
        ####### Horizontal forces ######
        rel_hori_vel = flow_velocity - xdot_list[iteration_step, 0] + xdot_list[iteration_step, 2] * (z_location_turbines)
        # Force and moment of drag force, F = 1/2. * rho * C * v^2 * A,
        Drag_force = 1/2. * C_drag_turbine * rho * A_hitbox * rel_hori_vel * abs(rel_hori_vel)
        Turbine_Force = 0.5 * rho * rel_hori_vel**3 * 0.25*np.pi*Diameter_turbine_impeller * 4 * Induction_factor * (1-Induction_factor)**2
        Drag_moment = Drag_force * (-z_location_bottom_floater)
        Turbine_Moment = - Turbine_Force * z_location_turbines

        ##### vertical forces #######
        Vertical_Flow_Force = 0
        Vertical_Flow_Moment = 0
        y_loc_coordinate = np.linspace(-FFTurbine_L / 2 + FFTurbine_L / 28,FFTurbine_L / 2 - FFTurbine_L / 28, 14)
        u_z= np.zeros(14)
        for j in range(14):
            u_z[j] = - xdot_list[iteration_step, 1]  - xdot_list[iteration_step, 2] * y_loc_coordinate[j]
            Vertical_Flow_Force += 1 / 2. * C_drag_turbine * rho * A_bottom_turbine / 14. * abs(u_z[j]) * u_z[j]
            Vertical_Flow_Moment += 1 / 2. * C_drag_turbine * rho * A_bottom_turbine / 14. * abs(u_z[j]) * u_z[j] * y_loc_coordinate[j] 



    N_matrix = 3
    # Empty list of forces for every step
    F = np.zeros((steps, N_matrix))

    # Fill list for every step
    F[iteration_step, 0] = Drag_force + Turbine_Force
    F[iteration_step, 1] = Vertical_Flow_Force
    F[iteration_step, 2] = Drag_moment + Vertical_Flow_Moment + Turbine_Moment


    return (F)


def Wind_Deck(segments, steps, Length_segment, z_location_deck, Wind_Distributed_Load_Deck,
              Three_Dimensional_Calculation):
    """"
    # Function:
# This function makes an estimation of the wind force on the deck, using the given load by the indonesian code

# Input:
# segments                          		= Number of segments in the bridge                      	(1)
# steps                             		= Number of steps that the dynamic model will perform   	(1)
# Length_segment							= Length of a single segment								(1)
# z_location_deck							= Z-coordinate of the middle deck							(1)
# Wind_Distributed_Load_Deck				= Distributed load by the wind on the deck					(1)
# Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)

# Output:
# F                 						= Force list for every step                                 ((steps) x N)
"""

    # Force and moment of wind on deck
    Wind_Deck_Force = Wind_Distributed_Load_Deck * Length_segment
    Wind_Deck_Moment = - z_location_deck * Wind_Deck_Force

    # Creating empty series of lists and fill it with force and moment for every segment
    if Three_Dimensional_Calculation == True:
        N_matrix = 6 * segments
    else:
        N_matrix = 3

    # Empty list of forces for every step
    F = np.zeros((steps, N_matrix))

    # Fill list for every step
    if Three_Dimensional_Calculation == True:
        for i in range(segments):
            F[:, 1 + 6 * i] = Wind_Deck_Force * np.ones(steps)
            F[:, 3 + 6 * i] = Wind_Deck_Moment * np.ones(steps)
    else:
        F[:, 0] = Wind_Deck_Force * np.ones(steps)
        F[:, 2] = Wind_Deck_Moment * np.ones(steps)

    return (F)


def Wind_Super_Structure(segments, steps, Length_segment, z_location_Super_Structure,
                         Wind_Distributed_Load_Super_Structure, Three_Dimensional_Calculation):
    """"

# Function:
# This function makes an estimation of the wind force on the superstructure, using the given load by the indonesian code

# Input:
# segments                          		= Number of segments in the bridge                      	(1)
# steps                             		= Number of steps that the dynamic model will perform   	(1)
# Length_segment							= Length of a single segment								(1)
# z_location_Super_Structure				= Z-coordinate of the middle of the super structure			(1)
# Wind_Distributed_Load_Super_Structure		= Distributed load by the wind on the super structure		(1)
# Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)

# Output:
# F                 						= Force list for every step                                 ((steps) x N)
"""

    # Force and moment of wind on superstructure
    Wind_SS_Force = Wind_Distributed_Load_Super_Structure * Length_segment
    Wind_SS_Moment = - z_location_Super_Structure * Wind_SS_Force

    # Creating empty series of lists and fill it with force and moment for every segment
    if Three_Dimensional_Calculation == True:
        N_matrix = 6 * segments
    else:
        N_matrix = 3

    F = np.zeros((steps, N_matrix))

    if Three_Dimensional_Calculation == True:
        for i in range(segments):
            F[:, 1 + 6 * i] = Wind_SS_Force * np.ones(steps)
            F[:, 3 + 6 * i] = Wind_SS_Moment * np.ones(steps)
    else:
        F[:, 0] = Wind_SS_Force * np.ones(steps)
        F[:, 2] = Wind_SS_Moment * np.ones(steps)

    return (F)


def PendulumDrag(iteration_step, segments, steps, Length_segment, free_space_between_segments,
                 characteristics_small_floater, Length_Pendulum, Angle_Pendulum, Pendulum_hinge_top_location,
                 Diameter_pendulum, flow_velocity, rho, Three_Dimensional_Calculation):
    """"
    # Function:
# This function makes an estimation of the pendulum drag. it ignores the complex movement of the pendulum and assumes
# its static

# Input:
# iteration_step    						= Iteration step number                                 	(1)
# segments                          		= Number of segments in the bridge                      	(1)
# steps                             		= Number of steps that the dynamic model will perform   	(1)
# Length_segment							= Length of a single segment								(1)
# free_space_between_segments				= Free space between the small floaters between segments	(1)
# characteristics_small_floater     		= Number of, width and length of small floaters         	(3)
# Length_Pendulum							= List of lengths of pendulums								(2*segments-2)
# Angle_Pendulum							= List of the angles of all pendulums						(2*segments-2)
# Pendulum_hinge_top_location				= Y- and z-coordinate of the top hing of pendulum       	(2)
# Diameter_pendulum							= Diameter of the pendulums									(2*segments-2)
# flow_velocity                     		= Flow velocity of the fluid                            	(1)
# rho                               		= Density of the fluid                                  	(1)
# Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)

# Output:
# F                 						= Force list for every step                                 ((steps) x N)
"""

    N_small_floater, W_small_floater, L_small_floater = characteristics_small_floater

    dist_pendulums = Length_segment - 2 * 0.5 * W_small_floater - free_space_between_segments  # m

    if Three_Dimensional_Calculation == True:
        N_matrix = 6 * segments
    else:
        N_matrix = 3

    F = np.zeros((steps, N_matrix))

    # force by pendulum on segments, total drag force devided by two as half gets directly to the foundation
    Pendulum_Specific_Force = np.zeros(2 * segments - 2)
    for i in range(2 * segments - 2):
        Pendulum_Specific_Force[i] = 0.5 * (1 / 2. * (0.8 * np.sin(Angle_Pendulum[i]) + 0.2) * (Diameter_pendulum[i] *
                                                                                                Length_Pendulum[
                                                                                                    i] * np.sin(
                    Angle_Pendulum[i])) * flow_velocity *
                                            abs(flow_velocity) * rho)
        # print (Angle_Pendulum[i], Diameter_pendulum[i], flow_velocity,rho,Length_Pendulum[i],0.8 * np.sin(Angle_Pendulum[i]) + 0.2)

    Pendulum_Force = np.zeros(segments)
    Pendulum_Moment_x = np.zeros(segments)
    Pendulum_Moment_z = np.zeros(segments)

    for i in range(segments):
        if Three_Dimensional_Calculation == False:
            if i == 0:
                continue
            if i == segments - 1:
                continue

        if i == 0:
            Pendulum_Force[i] = Pendulum_Specific_Force[0]
            Pendulum_Moment_x[i] = - Pendulum_Specific_Force[0] * Pendulum_hinge_top_location[1]
            Pendulum_Moment_z[i] = Pendulum_Specific_Force[0] * 0.5 * dist_pendulums

        elif i == segments - 1:
            Pendulum_Force[i] = Pendulum_Specific_Force[-1]
            Pendulum_Moment_x[i] = - Pendulum_Specific_Force[-1] * Pendulum_hinge_top_location[1]
            Pendulum_Moment_z[i] = - Pendulum_Specific_Force[-1] * 0.5 * dist_pendulums


        else:
            Pendulum_Force[i] = Pendulum_Specific_Force[2 * i - 1] + Pendulum_Specific_Force[2 * i]
            Pendulum_Moment_x[i] = - Pendulum_Specific_Force[2 * i - 1] * Pendulum_hinge_top_location[1] - \
                                   Pendulum_Specific_Force[2 * i] * Pendulum_hinge_top_location[1]
            Pendulum_Moment_z[i] = - Pendulum_Specific_Force[2 * i - 1] * 0.5 * dist_pendulums + \
                                   Pendulum_Specific_Force[2 * i] * 0.5 * dist_pendulums

    if Three_Dimensional_Calculation == True:
        for i in range(segments):
            F[iteration_step, 1 + 6 * i] = Pendulum_Force[i]
            F[iteration_step, 3 + 6 * i] = Pendulum_Moment_x[i]
            F[iteration_step, 5 + 6 * i] = Pendulum_Moment_z[i]

    else:
        F[iteration_step, 0] = Pendulum_Force[1]
        F[iteration_step, 2] = Pendulum_Moment_x[1]
    return (F)


def FloaterInertia(iteration_step, Total_time, x_list, steps, Length_segment,
                   free_space_between_segments, characteristics_small_floater,
                   characteristics_large_floater, M_segment, z_location_bottom_floater, Tidal_level, flow_velocity,
                   Wave_hlp, depth, rho, g, M_added_floater, gridsize, Current_Force, Approaching_Wave_Field, draught,C_inertia):
    """"# Function:
# This function makes an estimation of the force by waves, ignores the drag aspect as KC < 3. Only Hydro and Krylov force

# Input:
# iteration_step    						= Iteration step number                                 	(1)
# Total_time								= Total time that is simulated by the model					(1)
# x_list            						= Storage of location as function of time               	((steps + 1) x N)
# xddot_list        						= Storage of acceleration as function of time           	((steps) x N)
# segments                          		= Number of segments in the bridge                      	(1)
# steps                             		= Number of steps that the dynamic model will perform   	(1)
# characteristics_small_floater     		= Number of, width and length of small floaters         	(3)
# characteristics_large_floater     		= Number of, width and length of large floaters         	(3)
# M_segment                         		= Mass of a single segment                              	(1)
# z_location_bottom_floater         		= Z-coordinate of the bottom side of the floaters       	(1)
# Tidal_level								= Tidal level to be evaluated								(1)
# flow_velocity                     		= Flow velocity of the fluid                            	(1)
# Wave_hlp									= Regular wave height, length and period					(3)
# depth										= Local water depth											(1)
# rho                               		= Density of the fluid                                  	(1)
# g											= Gravitational acceleration								(1)
# Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)
# Current_Force								= Option to implement force by current on floaters			(T/F)


# Output:
# F                 						= Force list for every step                                 ((steps) x N)
"""
    x_list_0 = x_list[iteration_step, 0]
    x_list_1 = x_list[iteration_step, 1]
    x_list_2 = x_list[iteration_step, 2]

    # Extract data
    gridsize_h, gridsize_v = gridsize
    Wave_height, Wave_length, Wave_period = Wave_hlp

    
    # Fix wave direction, as negative waves are entered for negative direction
    if Wave_height < 0:
        Wave_height = - Wave_height
        Wave_direction = -1
    else:
        Wave_direction = 1

    if Current_Force == False:
        flow_velocity = 0

    # Wave parameters
    Wave_k = 2 * np.pi / Wave_length
    Wave_w = 2 * np.pi / Wave_period
    a = Wave_height / 2.

    # Depth fix, include additional water level due to tides
    depth = depth + Tidal_level

    # Create empty matrix and force array collection
    N_matrix = 3
    F = np.zeros(N_matrix)

    # seperate data from arrays which were imported
    N_small_floater, W_small_floater, L_small_floater = characteristics_small_floater
    N_large_floater, W_large_floater, L_large_floater = characteristics_large_floater

    # Horizontal area of floaters, including the tips
    A_small_floater = W_small_floater * L_small_floater + 2 * 1 / 2. * W_small_floater * W_small_floater  # m2
    A_large_floater = W_large_floater * L_large_floater + 2 * 1 / 2. * W_large_floater * 0.5 * W_large_floater  # m2

    # correction for horizontal position of the bridge as this will delay wave impact
    y_bridge = x_list_0

    # gridsize
    L_grid_small = L_small_floater + 2 * W_small_floater
    L_grid_large = L_large_floater + W_large_floater

    Grid_Hor_small = int(L_grid_small / gridsize_h)
    Grid_Hor_large = int(L_grid_large / gridsize_h)

    if Grid_Hor_large < 10:
        Grid_Hor_large = 10

    if Grid_Hor_small < 10:
        Grid_Hor_small = 10

    gridsize_h_small = L_grid_small / Grid_Hor_small
    gridsize_h_large = L_grid_large / Grid_Hor_large

    # Wave height as function of y location, empty array
    eta_small = np.zeros(Grid_Hor_small)
    eta_large = np.zeros(Grid_Hor_large)

    time = np.linspace(0, Total_time, steps)[iteration_step]

    # define horizontal grid y coordinates
    y_space_small = np.linspace(-L_grid_small / 2. + gridsize_h_small / 2., L_grid_small / 2. - gridsize_h_small / 2.,
                                Grid_Hor_small)
    y_space_large = np.linspace(-L_grid_large / 2. + gridsize_h_large / 2., L_grid_large / 2. - gridsize_h_large / 2.,
                                Grid_Hor_large)

    # Tricks to save time
    Constant_01 = Wave_w * time + flow_velocity * time / Wave_length * 2 * np.pi
    Constant_02 = Wave_k * a ** 2 * np.cosh(Wave_k * depth) / (4 * (np.sinh(Wave_k * depth)) ** 3) * (
                2 + np.cosh(2 * Wave_k * depth))
    Constant_03 = 3 / 4. * rho * g * a ** 2 * Wave_k * np.tanh(Wave_k * depth) / (np.sinh(Wave_k * depth)) ** 3
    Constant_04 = 1 / 4. * rho * g * a ** 2 * Wave_k * np.tanh(Wave_k * depth) / (np.sinh(Wave_k * depth)) ** 2
    Constant_05 = 6 / 4. * (Wave_w + flow_velocity * Wave_k) * Wave_w * a ** 2 * Wave_k
    Constant_06 = - (Wave_w + flow_velocity * Wave_k) * Wave_w * a
    Constant_07 = np.sinh(Wave_k * depth)
    Constant_08 = (np.cosh(Wave_k * depth)) ** 3 * np.sinh(Wave_k * depth)
    Constant_09 = np.sinh(Wave_k * depth)
    Constant_10 = Wave_direction * Wave_k
    Constant_11 = gridsize_h_small * gridsize_v * rho * N_small_floater
    Constant_12 = gridsize_h_large * gridsize_v * rho * N_large_floater
    Constant_13 = rho * g
    Constant_14 = gridsize_h_small * N_small_floater
    Constant_15 = gridsize_h_large * N_large_floater
    Constant_16 = 1. / np.cosh(Wave_k * depth)


    ######  WAVE HEIGHT #######
    
    # wave height as function of y for every segment

    eta_small = a * np.cos(Constant_01 - Constant_10 * (y_space_small + y_bridge)) + Constant_02 * np.cos(2 * (Constant_01 - Constant_10 * (y_space_small + y_bridge)))

    #print(y_bridge)
    #The y_bridge gets instable for larger stiffness
    eta_large = a * np.cos(Constant_01 - Constant_10 * (y_space_large + y_bridge)) + Constant_02 * np.cos(2 * (Constant_01 - Constant_10 * (y_space_large + y_bridge)))

    
    # Max vertical displacement of segment
    max_xlist = 0
    if max_xlist < x_list_1:
        max_xlist = x_list_1
    
    
    # Define vertical grid z coordinate
    H_grid = draught + np.max([np.max(eta_small), np.max(eta_large)]) + max_xlist + gridsize_v * 2

    Grid_Ver = int(H_grid / gridsize_v)

    if Grid_Ver < 20:
        Grid_Ver = 20

    gridsize_v = H_grid / Grid_Ver

    # empty matrix, will contain ones and zeros, wave present or not?
    Wave_present_small = np.zeros((Grid_Hor_small, Grid_Ver))
    Wave_present_large = np.zeros((Grid_Hor_large, Grid_Ver))

    # width of floaters as function of y
    Width_small = W_small_floater * np.ones(Grid_Hor_small)
    Width_large = W_large_floater * np.ones(Grid_Hor_large)

    i = 0
    while True:
        if (i+0.5) * gridsize_h_small > W_small_floater:
            break
        Width_small[i] = (i+0.5) * gridsize_h_small
        Width_small[-i-1] = (i + 0.5) * gridsize_h_small
        i += 1


    i = 0
    while True:
        if (i+0.5) * gridsize_h_large > W_large_floater /2.:
            break
        Width_large[i] = 2*(i+0.5) * gridsize_h_large
        Width_large[-i-1] = 2*(i + 0.5) * gridsize_h_large
        i += 1


    # Empty force array (for segments)
    V_Force_pressure = 0
    V_Moment_pressure = 0

    V_Force_hydro_krylov = 0
    V_Moment_hydro_krylov = 0

    H_Force_hydro_krylov = 0
    H_Moment_hydro_krylov = 0

    # empty array for pressure at bottom
    pressure_small = np.zeros(Grid_Hor_small)
    pressure_large = np.zeros(Grid_Hor_large)

    # z coordinate as function of y. considers draught, heave and roll
    z_large = np.zeros(Grid_Hor_large)
    z_large = - draught + x_list_1 + y_space_small * x_list_2

    z_small = np.zeros(Grid_Hor_small)
    z_small = - draught + x_list_1 + y_space_small * x_list_2


    ####### PRESSURE #######
    # Socond stokes pressure equations.
    for i in range(Grid_Hor_small):
        if eta_small[i] < z_small[i]:
            pressure_small[i] = 0
        else:
            if z_small[i] > 0:
                pressure_small[i] = - Constant_13 * (eta_small[i] - z_small[i])

            else:
                pressure_small[i] = - Constant_13 * (z_small[i]) + Constant_13 * a * np.cosh(Wave_k * (depth + z_small[i])) * Constant_16 * np.cos(Constant_01 - Constant_10 * (y_space_small[i] + y_bridge)) + Constant_03 * (np.cosh(2 * Wave_k * (depth + z_small[i])) / (Constant_09) ** 3 - 1 / 3.) * np.sin(2 * (Constant_01 - Constant_10 * (y_space_small[i] + y_bridge))) - Constant_04 * (np.cosh(2 * Wave_k * (depth + z_small[i])) - 1)

    if Approaching_Wave_Field == True:
        NumberOfWavesCancelled = int(np.ceil(0.3 * Total_time / Wave_period + flow_velocity * 0.3 * Total_time / Wave_length) + 1)
        Cancelled_length = NumberOfWavesCancelled * Wave_length

        if Wave_direction == 1:
            if (iteration_step / steps * Total_time / Wave_period + flow_velocity * iteration_step / steps * Total_time / Wave_length) * Wave_length - y_space_small < Cancelled_length + 0.25 * Wave_length:
                pressure_small = - z_small * Constant_13
            elif (iteration_step / steps * Total_time / Wave_period + flow_velocity * iteration_step / steps * Total_time / Wave_length) * Wave_length - y_space_small < Cancelled_length + 0.5 * Wave_length:
                pressure_small = - z_small * Constant_13 * (0.5 - 0.5 * np.cos(2 * (Wave_w * iteration_step / steps * Total_time + flow_velocity * iteration_step / steps * Total_time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (
                        y_space_small + y_bridge))))  + pressure_small * (0.5 + 0.5 * np.cos(2 * (Wave_w * iteration_step / steps * Total_time + flow_velocity * iteration_step / steps * Total_time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (y_space_small + y_bridge))))
        else:
            if (iteration_step / steps * Total_time / Wave_period + flow_velocity * iteration_step / steps * Total_time / Wave_length) * Wave_length * Wave_direction - y_space_small > Cancelled_length * Wave_direction - 0.25 * Wave_length:
                pressure_small = - z_small * Constant_13
            elif (iteration_step / steps * Total_time / Wave_period + flow_velocity * iteration_step / steps * Total_time / Wave_length) * Wave_length * Wave_direction - y_space_small > Cancelled_length * Wave_direction - 0.5 * Wave_length:
                pressure_small = - z_small * Constant_13 * (0.5 - 0.5 * np.cos(2 * (Wave_w * iteration_step / steps * Total_time + flow_velocity * iteration_step / steps * Total_time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (y_space_small + y_bridge)))) + pressure_small  * (0.5 + 0.5 * np.cos(2 * (Wave_w * iteration_step / steps * Total_time + flow_velocity * iteration_step / steps * Total_time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (y_space_small + y_bridge))))

    for i in range(Grid_Hor_large):
        if eta_large[i] < z_large[i]:
            pressure_large[i] = 0
        else:
            if z_large[i] > 0:
                pressure_large[i] = - Constant_13 * (eta_large[i] - z_large[i])

            else:
                pressure_large[i] = - Constant_13 * (z_large[i]) + Constant_13 * a * np.cosh(Wave_k * (depth + z_large[i])) * Constant_16 * np.cos(Constant_01 - Constant_10 * (y_space_large[i] + y_bridge)) + Constant_03 * (np.cosh(2 * Wave_k * (depth + z_large[i])) / (Constant_09) ** 3 - 1 / 3.) * np.sin(
                    2 * (Constant_01 - Constant_10 * (y_space_large[i] + y_bridge))) - Constant_04 * (np.cosh(2 * Wave_k * (depth + z_large[i])) - 1)


    if Approaching_Wave_Field == True:
        NumberOfWavesCancelled = int(np.ceil(0.3 * Total_time / Wave_period + flow_velocity * 0.3 * Total_time / Wave_length) + 1)
        Cancelled_length = NumberOfWavesCancelled * Wave_length
        
        if Wave_direction == 1:
            if (iteration_step / steps * Total_time / Wave_period + flow_velocity * iteration_step / steps * Total_time / Wave_length) * Wave_length - y_space_large[i] < Cancelled_length + 0.25 * Wave_length:
                pressure_large = - z_large * Constant_13
            elif (iteration_step / steps * Total_time / Wave_period + flow_velocity * iteration_step / steps * Total_time / Wave_length) * Wave_length - y_space_large[i] < Cancelled_length + 0.5 * Wave_length:
                pressure_large = - z_large * Constant_13 * (0.5 - 0.5 * np.cos(2 * (Wave_w * iteration_step / steps * Total_time + flow_velocity * iteration_step / steps * Total_time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (y_space_large[i] + y_bridge))))  + pressure_large[i] * (0.5 + 0.5 * np.cos(2 * (Wave_w * iteration_step / steps * Total_time + flow_velocity * iteration_step / steps * Total_time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (y_space_large + y_bridge))))
        else:
            if (iteration_step / steps * Total_time / Wave_period + flow_velocity * iteration_step / steps * Total_time / Wave_length) * Wave_length * Wave_direction - y_space_large[i] > Cancelled_length * Wave_direction - 0.25 * Wave_length:
                pressure_large = - z_large * Constant_13
            elif (iteration_step / steps * Total_time / Wave_period + flow_velocity * iteration_step / steps * Total_time / Wave_length) * Wave_length * Wave_direction - y_space_large[i] > Cancelled_length * Wave_direction - 0.5 * Wave_length:
                pressure_large = - z_large * Constant_13 * (0.5 - 0.5 * np.cos(2 * (Wave_w * iteration_step / steps * Total_time + flow_velocity * iteration_step / steps * Total_time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (y_space_large[i] + y_bridge)))) + pressure_large[i]  * (0.5 + 0.5 * np.cos(2 * (Wave_w * iteration_step / steps * Total_time + flow_velocity * iteration_step / steps * Total_time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (y_space_large + y_bridge))))


    # Calculate forces by multiplying pressure with area
    V_Force_pressure_small = np.zeros(Grid_Hor_small)
    V_Force_pressure_small = Constant_14 * Width_small * pressure_small
    V_Force_pressure_large = np.zeros(Grid_Hor_large)
    V_Force_pressure_large = Constant_15 * Width_large * pressure_large
    V_Force_pressure = np.sum(V_Force_pressure_small) + np.sum(V_Force_pressure_large)
    
    V_Moment_pressure_small = np.zeros(Grid_Hor_small)
    V_Moment_pressure_small = Constant_14 * Width_small * pressure_small * y_space_small
    V_Moment_pressure_large = np.zeros(Grid_Hor_large)
    V_Moment_pressure_large = Constant_15 * Width_large * pressure_large * y_space_large
    V_Moment_pressure = np.sum(V_Moment_pressure_small) + np.sum(V_Moment_pressure_large)



    # Fill WaterPresent? matrix with ones and zeros
    for j in range(Grid_Hor_small):
        for k in range(Grid_Ver):
            if eta_small[j] + draught - x_list_1 > (k + 0.5) * gridsize_v:
                Wave_present_small[j, k] = 1

    for j in range(Grid_Hor_large):
        for k in range(Grid_Ver):
            if eta_large[j] + draught - x_list_1 > (k + 0.5) * gridsize_v:
                Wave_present_large[j, k] = 1



    # empty matrices for horizontal and vertical water particle acceleration
    udot_y_small = np.zeros((Grid_Hor_small, Grid_Ver))
    udot_y_large = np.zeros((Grid_Hor_large, Grid_Ver))

    udot_z_small = np.zeros((Grid_Hor_small, Grid_Ver))
    udot_z_large = np.zeros((Grid_Hor_large, Grid_Ver))

    # Fill those just defined matrices
    for j in range(Grid_Ver):

        # check if water is present
        z_large = np.zeros(Grid_Hor_large)
        z_large = - draught + x_list_1 + y_space_small * x_list_2 + (j + 0.5) * gridsize_v

        z_small = np.zeros(Grid_Hor_small)
        z_small = - draught + x_list_1 + y_space_small * x_list_2 + (j + 0.5) * gridsize_v

    # Fill those just defined matrices
    for j in range(Grid_Ver):

        # check if water is present
        z_large = np.zeros(Grid_Hor_large)
        for k in range(Grid_Hor_large):
            z_large[k] = - draught + x_list[iteration_step, 1] + y_space_small[k] * x_list[
                iteration_step, 2] + (j + 0.5) * gridsize_v

        z_small = np.zeros(Grid_Hor_small)
        for k in range(Grid_Hor_small):
            z_small[k] = - draught + x_list[iteration_step, 1] + y_space_small[k] * x_list[iteration_step, 2] + (j + 0.5) * gridsize_v

        # Fill
        for k in range(Grid_Hor_small):

            if Wave_present_small[k, j] == 1:
                udot_y_small[k, j] = Constant_06 * np.cosh(Wave_k * (z_small[k] + depth)) / Constant_07 * np.sin(Constant_01 - Constant_10 * (y_space_small[k] + y_bridge)) - Constant_05 * np.cosh(2 * Wave_k * (z_small[k] + depth)) / (Constant_08) * np.sin(2 * (Constant_01 - Constant_10 * (y_space_small[k] + y_bridge)))
                udot_z_small[k, j] = Constant_06 * np.sinh(Wave_k * (z_small[k] + depth)) / Constant_07 * np.cos(Constant_01 - Constant_10 * (y_space_small[k] + y_bridge)) - Constant_05 * np.sinh(2 * Wave_k * (z_small[k] + depth)) / (Constant_08) * np.cos(2 * (Constant_01 - Constant_10 * (y_space_small[k] + y_bridge)))

                if Approaching_Wave_Field == True:
                    NumberOfWavesCancelled = int(np.ceil(0.3 * Total_time / Wave_period + flow_velocity * 0.3 * Total_time / Wave_length) + 1)
                    Cancelled_length = NumberOfWavesCancelled * Wave_length

                    if Wave_direction == 1:
                        if (iteration_step / steps * Total_time / Wave_period + flow_velocity * iteration_step / steps * Total_time / Wave_length) * Wave_length - y_space_small < Cancelled_length + 0.25 * Wave_length:
                            udot_y_small[k, j] = 0
                        elif (iteration_step / steps * Total_time / Wave_period + flow_velocity * iteration_step / steps * Total_time / Wave_length) * Wave_length - y_space_small < Cancelled_length + 0.5 * Wave_length:
                            udot_y_small[k, j] = udot_y_small[k, j]  * (0.5 + 0.5 * np.cos(2 * (Wave_w * iteration_step / steps * Total_time + flow_velocity * iteration_step / steps * Total_time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (y_space_small[k] + y_bridge))))
                    else:
                        if (iteration_step / steps * Total_time / Wave_period + flow_velocity * iteration_step / steps * Total_time / Wave_length) * Wave_length * Wave_direction - y_space_small > Cancelled_length * Wave_direction - 0.25 * Wave_length:
                            udot_y_small[k, j] = 0
                        elif (iteration_step / steps * Total_time / Wave_period + flow_velocity * iteration_step / steps * Total_time / Wave_length) * Wave_length * Wave_direction - y_space_small > Cancelled_length * Wave_direction - 0.5 * Wave_length:
                            udot_y_small[k, j] = udot_y_small[k, j] * (0.5 + 0.5 * np.cos(2 * (Wave_w * iteration_step / steps * Total_time + flow_velocity * iteration_step / steps * Total_time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (y_space_small[k] + y_bridge))))


                    if Wave_direction == 1:
                        if (iteration_step / steps * Total_time / Wave_period + flow_velocity * iteration_step / steps * Total_time / Wave_length) * Wave_length - y_space_small < Cancelled_length + 0.25 * Wave_length:
                            udot_z_small[k, j] = 0
                        elif (iteration_step / steps * Total_time / Wave_period + flow_velocity * iteration_step / steps * Total_time / Wave_length) * Wave_length - y_space_small < Cancelled_length + 0.5 * Wave_length:
                            udot_z_small[k, j] = udot_z_small[k, j]  * (0.5 + 0.5 * np.cos(2 * (Wave_w * iteration_step / steps * Total_time + flow_velocity * iteration_step / steps * Total_time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (y_space_small[k] + y_bridge))))
                    else:
                        if (iteration_step / steps * Total_time / Wave_period + flow_velocity * iteration_step / steps * Total_time / Wave_length) * Wave_length * Wave_direction - y_space_small > Cancelled_length * Wave_direction - 0.25 * Wave_length:
                            udot_z_small[k, j] = 0
                        elif (iteration_step / steps * Total_time / Wave_period + flow_velocity * iteration_step / steps * Total_time / Wave_length) * Wave_length * Wave_direction - y_space_small > Cancelled_length * Wave_direction - 0.5 * Wave_length:
                            udot_z_small[k, j] = udot_z_small[k, j] * (0.5 + 0.5 * np.cos(2 * (Wave_w * iteration_step / steps * Total_time + flow_velocity * iteration_step / steps * Total_time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (y_space_small[k] + y_bridge))))


        for k in range(Grid_Hor_large):

            if Wave_present_large[k, j] == 1:
                udot_y_large[k, j] = Constant_06 * np.cosh(Wave_k * (z_large[k] + depth)) / Constant_07 * np.sin(Constant_01 - Constant_10 * (y_space_large[k] + y_bridge)) - Constant_05 * np.cosh(2 * Wave_k * (z_large[k] + depth)) / (Constant_08) * np.sin(2 * (Constant_01 - Constant_10 * (y_space_large[k] + y_bridge)))
                udot_z_large[k, j] = Constant_06 * np.sinh(Wave_k * (z_large[k] + depth)) / Constant_07 * np.cos(Constant_01 - Constant_10 * (y_space_large[k] + y_bridge)) - Constant_05 * np.sinh(2 * Wave_k * (z_large[k] + depth)) / (Constant_08) * np.cos(2 * (Constant_01 - Constant_10 * (y_space_large[k] + y_bridge)))

                if Approaching_Wave_Field == True:
                    NumberOfWavesCancelled = int(np.ceil(0.3 * Total_time / Wave_period + flow_velocity * 0.3 * Total_time / Wave_length) + 1)
                    Cancelled_length = NumberOfWavesCancelled * Wave_length

                    if Wave_direction == 1:
                        if (iteration_step / steps * Total_time / Wave_period + flow_velocity * iteration_step / steps * Total_time / Wave_length) * Wave_length - y_space_large < Cancelled_length + 0.25 * Wave_length:
                            udot_y_large[k, j] = 0
                        elif (iteration_step / steps * Total_time / Wave_period + flow_velocity * iteration_step / steps * Total_time / Wave_length) * Wave_length - y_space_large < Cancelled_length + 0.5 * Wave_length:
                            udot_y_large[k, j] = udot_y_large[k, j] * (0.5 + 0.5 * np.cos(2 * (Wave_w * iteration_step / steps * Total_time + flow_velocity * iteration_step / steps * Total_time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (y_space_large[k] + y_bridge))))
                    else:
                        if (iteration_step / steps * Total_time / Wave_period + flow_velocity * iteration_step / steps * Total_time / Wave_length) * Wave_length * Wave_direction - y_space_large > Cancelled_length * Wave_direction - 0.25 * Wave_length:
                            udot_y_large[k, j] = 0
                        elif (iteration_step / steps * Total_time / Wave_period + flow_velocity * iteration_step / steps * Total_time / Wave_length) * Wave_length * Wave_direction - y_space_large > Cancelled_length * Wave_direction - 0.5 * Wave_length:
                            udot_y_large[k, j] =  udot_y_large[k, j]  * (0.5 + 0.5 * np.cos(2 * (Wave_w * iteration_step / steps * Total_time + flow_velocity * iteration_step / steps * Total_time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (y_space_large[k] + y_bridge))))

                    if Wave_direction == 1:
                        if (iteration_step / steps * Total_time / Wave_period + flow_velocity * iteration_step / steps * Total_time / Wave_length) * Wave_length - y_space_large < Cancelled_length + 0.25 * Wave_length:
                            udot_z_large[k, j] = 0
                        elif (iteration_step / steps * Total_time / Wave_period + flow_velocity * iteration_step / steps * Total_time / Wave_length) * Wave_length - y_space_large < Cancelled_length + 0.5 * Wave_length:
                            udot_z_large[k, j] = udot_z_large[k, j]  * (0.5 + 0.5 * np.cos(2 * (Wave_w * iteration_step / steps * Total_time + flow_velocity * iteration_step / steps * Total_time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (y_space_large[k] + y_bridge))))
                    else:
                        if (iteration_step / steps * Total_time / Wave_period + flow_velocity * iteration_step / steps * Total_time / Wave_length) * Wave_length * Wave_direction - y_space_large > Cancelled_length * Wave_direction - 0.25 * Wave_length:
                            udot_z_large[k, j] = 0
                        elif (iteration_step / steps * Total_time / Wave_period + flow_velocity * iteration_step / steps * Total_time / Wave_length) * Wave_length * Wave_direction - y_space_large > Cancelled_length * Wave_direction - 0.5 * Wave_length:
                            udot_z_large[k, j] = udot_z_large[k, j] * (0.5 + 0.5 * np.cos(2 * (Wave_w * iteration_step / steps * Total_time + flow_velocity * iteration_step / steps * Total_time / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (y_space_large[k] + y_bridge))))


    C_added_vertical = M_added_floater[1, 1] / draught / (A_small_floater * N_small_floater + A_large_floater * N_large_floater) / rho
    C_added_horizontal = M_added_floater[0, 0] / draught / (A_small_floater * N_small_floater + A_large_floater * N_large_floater) / rho


    #Constant_11 = gridsize_h_small * gridsize_v * rho * N_small_floater [m]
    for j in range(Grid_Ver):
        for k in range(Grid_Hor_small):
            V_Force_hydro_krylov += Constant_11 * Width_small[k] * (C_added_vertical + 1 ) * (udot_z_small[k, j])
            H_Force_hydro_krylov += Constant_11 * Width_small[k] * (C_added_horizontal + 1 ) * (udot_y_small[k, j])

            V_Moment_hydro_krylov += Constant_11 * Width_small[k] * (C_added_vertical + 1 ) * (udot_z_small[k, j]) * y_space_small[k]
            H_Moment_hydro_krylov += -Constant_11 * Width_small[k] * (C_added_horizontal + 1 ) * (udot_y_small[k, j]) * (z_location_bottom_floater + (j + 0.5) * gridsize_v)

        for k in range(Grid_Hor_large):
            V_Force_hydro_krylov += Constant_12 * Width_large[k] * (C_added_vertical + 1 ) * (udot_z_large[k, j])
            H_Force_hydro_krylov += Constant_12 * Width_large[k] * (C_added_horizontal + 1 ) * (udot_y_large[k, j])

            V_Moment_hydro_krylov += Constant_12 * Width_large[k] * (C_added_vertical + 1 ) * (udot_z_large[k, j]) * y_space_large[k]
            H_Moment_hydro_krylov += -Constant_12 * Width_large[k] * (C_added_horizontal + 1 ) * (udot_y_large[k, j]) * (z_location_bottom_floater + (j + 0.5) * gridsize_v)


    # add gravity force, so constant druaght > z = 0
    V_Force_gravity = - Constant_13 * (A_large_floater * N_large_floater + A_small_floater * N_small_floater) * draught #Verified


    # Add found values to forcing array collection
    F[0] = C_inertia*H_Force_hydro_krylov
    F[1] = C_inertia*V_Force_hydro_krylov + V_Force_pressure + V_Force_gravity
    F[2] = C_inertia*H_Moment_hydro_krylov + C_inertia*V_Moment_hydro_krylov + V_Moment_pressure


    return (F)


def TurbineInertia(iteration_step, x_list, Wave_hlp, Current_Force, depth, Tidal_level, draught, flow_velocity, M_added_tur_hori,
                   M_added_tur_vert, z_location_bottom_floater, rho, Total_time, steps,Turbine_dimensions_HWL, C_inertia):
    
    
    # iteration_step, Total_time, , steps, Length_segment,
    #                free_space_between_segments, characteristics_small_floater,
    #                characteristics_large_floater, M_segment, z_location_bottom_floater, Tidal_level, flow_velocity,
    #                Wave_hlp, depth, rho, g, M_added_floater, gridsize, Current_Force, Approaching_Wave_Field):
    """"# Function:
# This function makes an estimation of the force by waves, ignores the drag aspect as KC < 3. Only Hydro and Krylov force

# Input:
# iteration_step    						= Iteration step number                                 	(1)
# Total_time								= Total time that is simulated by the model					(1)
# x_list            						= Storage of location as function of time               	((steps + 1) x N)
# xddot_list        						= Storage of acceleration as function of time           	((steps) x N)
# segments                          		= Number of segments in the bridge                      	(1)
# steps                             		= Number of steps that the dynamic model will perform   	(1)
# characteristics_small_floater     		= Number of, width and length of small floaters         	(3)
# characteristics_large_floater     		= Number of, width and length of large floaters         	(3)
# M_segment                         		= Mass of a single segment                              	(1)
# z_location_bottom_floater         		= Z-coordinate of the bottom side of the floaters       	(1)
# Tidal_level								= Tidal level to be evaluated								(1)
# flow_velocity                     		= Flow velocity of the fluid                            	(1)
# Wave_hlp									= Regular wave height, length and period					(3)
# depth										= Local water depth											(1)
# rho                               		= Density of the fluid                                  	(1)
# g											= Gravitational acceleration								(1)
# Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)
# Current_Force								= Option to implement force by current on floaters			(T/F)


# Output:
# F                 						= Force list for every step                                 ((steps) x N)
"""
    x_sway = x_list[iteration_step, 0]
    x_heave = x_list[iteration_step, 1]
    x_roll = x_list[iteration_step, 2]

    # Extract data
    H, L, T = Wave_hlp
    # seperate data from arrays which were imported
    FFTurbine_H, FFTurbine_W, FFTurbine_L, Turbine_depth, Tur_grid_size, Turbine_Sub_Volume, N_turbines, FFTur_evalf_loc = Turbine_dimensions_HWL
    
    # Fix wave direction, as negative waves are entered for negative direction
    if H < 0:
        H = - H
        Wave_direction = -1
    else:
        Wave_direction = 1

    u = flow_velocity
    if Current_Force == False:
        u = 0

    # Wave parameters
    k = 2 * np.pi / L
    w = 2 * np.pi / T
    a = H / 2.

    # Depth fix, include additional water level due to tides
    d = depth + Tidal_level
    
    # correction for horizontal position of the bridge as this will delay wave impact
    y_bridge = x_sway

    # Create empty matrix and force array collection
    N_matrix = 3
    F = np.zeros(N_matrix)


    # gridsize
    Grid_Hor = int(FFTurbine_L/Tur_grid_size) #estimation of the number of grids
    if Grid_Hor < 10: #Guarantee to have at least 10 gridscells
        Grid_Hor = 10
    gridsize_h = FFTurbine_L / Grid_Hor #Real horizontal grid size
    # define horizontal grid y coordinates
    y = np.linspace(-FFTurbine_L / 2. + gridsize_h / 2., FFTurbine_L / 2. - gridsize_h / 2.,Grid_Hor)

    # empty array
    t = np.linspace(0, Total_time, steps)[iteration_step]
    
    # Tricks to save time
    C_01 = w * t + u * t * k
    # C_02 = k * a ** 2 * np.cosh(k * d) / (4 * (np.sinh(k * d)) ** 3) * (2 + np.cosh(2 * k * d))
    C_05 = 6 / 4. * (w + u * k) * w * a ** 2 * k
    C_06 = - (w + u * k) * w * a
    C_07 = np.sinh(k * d)
    C_08 = (np.cosh(k * d)) ** 3 * np.sinh(k * d)
    C_10 = Wave_direction * k

    # print('1', C_01, '5', C_05, '6', C_06, '7', C_07, '8', C_08, '10', C_10)


    ######  WAVE HEIGHT #######
    # wave height as function of y for every segment
    # eta = np.zeros(Grid_Hor)
    # eta = a * np.cos(C_01 - C_10 * (y + y_bridge)) + C_02 * np.cos(2 * (C_01 - C_10 * (y + y_bridge)))

    # z coordinate as function of y. considers draught, heave and roll
    z = np.zeros(Grid_Hor)
    z = - abs(Turbine_depth) - abs(FFTur_evalf_loc) + x_heave + y * x_roll

    
    # Empty force array (for segments)
    V_Force_hydro_krylov = 0
    V_Moment_hydro_krylov = 0
    H_Force_hydro_krylov = 0
    H_Moment_hydro_krylov = 0
    udot_y = np.zeros(Grid_Hor)
    udot_z = np.zeros(Grid_Hor)  
    
    udot_y = C_06 * np.cosh(k * (z + d)) / C_07 * np.sin(C_01 - C_10 * (y + y_bridge)) - C_05 * np.cosh(2 * k * (z + d)) / (C_08) * np.sin(2 * (C_01 - C_10 * (y + y_bridge)))
    udot_z = C_06 * np.sinh(k * (z + d)) / C_07 * np.cos(C_01 - C_10 * (y + y_bridge)) - C_05 * np.sinh(2 * k * (z + d)) / (C_08) * np.cos(2 * (C_01 - C_10 * (y + y_bridge)))

    for k in range(Grid_Hor):
        H_Force_hydro_krylov += N_turbines * udot_y[k] * ((Turbine_Sub_Volume * rho/Grid_Hor) + (M_added_tur_hori/Grid_Hor))
        V_Force_hydro_krylov += N_turbines * udot_z[k] * ((Turbine_Sub_Volume * rho/Grid_Hor) + (M_added_tur_vert/Grid_Hor))
        
        H_Moment_hydro_krylov += -N_turbines * udot_y[k] * ((Turbine_Sub_Volume * rho/Grid_Hor) + (M_added_tur_hori/Grid_Hor)) * (-abs(z_location_bottom_floater)+abs(draught)-abs(Turbine_depth)-abs(FFTur_evalf_loc) + y[k] * x_roll)
        V_Moment_hydro_krylov += N_turbines * udot_z[k] * ((Turbine_Sub_Volume * rho/Grid_Hor) + (M_added_tur_vert/Grid_Hor)) * (y[k] + abs(-abs(z_location_bottom_floater)+abs(draught)-abs(Turbine_depth)-abs(FFTur_evalf_loc)) * x_roll)
        
    # Add found values to forcing array collection
    F[0] = C_inertia*H_Force_hydro_krylov
    F[1] = C_inertia*V_Force_hydro_krylov
    F[2] = C_inertia*H_Moment_hydro_krylov + C_inertia*V_Moment_hydro_krylov


    return (F)


def SwayPlateForces(iteration_step, x_list, xdot_list, Wave_hlp, Current_Force, depth, Tidal_level, flow_velocity, rho, Total_time, 
                    steps, Turbine_dimensions_HWL, Wave_Force, COG_to_surface, Sway_plate_char, C_inertia):


    # seperate data from arrays which were imported
    FFTurbine_H, FFTurbine_W, FFTurbine_L, Turbine_depth, Tur_grid_size, Turbine_Volume, N_turbines, FFTur_evalf_loc = Turbine_dimensions_HWL
    Sway_width, Sway_height, z_location_sway, C_drag_sway, M_added_sway_plate, sway_plate_num = Sway_plate_char
    x_sway = x_list[iteration_step, 0]
    x_heave = x_list[iteration_step, 1]
    x_roll = x_list[iteration_step, 2]
    xd_sway = xdot_list[iteration_step, 0]
#    xd_heave = xdot_list[iteration_step, 1]
    xd_roll = xdot_list[iteration_step, 2]
    u = flow_velocity

    A_hitbox = Sway_width * Sway_height
    Sway_drag_force = 0
    Sway_drag_moment = 0
    H_Force_hydro = 0
    H_Moment_hydro = 0
    N_matrix = 3
    F = np.zeros(N_matrix)


    if Current_Force == False:
        u = 0    

    if Wave_Force == True:
        t = np.linspace(0, Total_time, steps)[iteration_step]
        H, L, T = Wave_hlp

        # Fix wave direction, as negative waves are entered for negative direction
        if H < 0:
            H = - H
            Wave_direction = -1
        else:
            Wave_direction = 1
        # Wave parameters
        Wave_k = 2 * np.pi / L
        Wave_w = 2 * np.pi / T
        k = Wave_k
        w = Wave_w
        a = H / 2.
        # Depth fix, include additional water level due to tides
        d = depth + Tidal_level

        # Tricks to save time
        C_01 = w * t + u * t * k
        C_05 = 6 / 4. * (w + u * k) * w * a ** 2 * k
        C_06 = - (w + u * k) * w * a
        C_07 = np.sinh(k * d)
        C_08 = (np.cosh(k * d)) ** 3 * np.sinh(k * d)
        C_10 = Wave_direction * k

##### HORIZONTAL CONTRIBUTION #####
        gridsize = Sway_height / 14.
        z_coordinate = np.linspace(-Turbine_depth - FFTurbine_H - Sway_height + gridsize / 2., -Turbine_depth - FFTurbine_H - gridsize / 2., 14)
        z_loc = np.zeros(14)
        z_dist_cog = np.zeros(14)
        y_loc = np.zeros(14)
        for j in range(14):
            z_loc[j] = z_coordinate[j] + x_heave
            z_dist_cog[j] = abs(z_coordinate[j]) + abs(COG_to_surface)
            y_loc[j] = x_sway + x_roll * z_dist_cog[j]
             
        u_y_zcoordinate = np.zeros(14)
        udot_y_zcoordinate = np.zeros(14)
        
        for j in range(14):
            #Velocity of water particles
            u_y_zcoordinate[j] = (Wave_w * a * np.cosh(Wave_k * (z_loc[j] + d)) / np.sinh(Wave_k * d) * np.cos(
                Wave_w * t + u * t / L * 2 * np.pi - Wave_direction * Wave_k * y_loc[j]) + 3 / 4. * Wave_w * a ** 2 * Wave_k * np.cosh(
                2 * Wave_k * (z_loc[j] + d)) / ((np.cosh(Wave_k * d)) ** 3 * np.sinh(Wave_k * d)) * np.cos(
                2 * (Wave_w * t + u * t / L * 2 * np.pi - Wave_direction * Wave_k * y_loc[j]))
                + u - xd_sway - xd_roll * z_dist_cog[j])
            #Drag force
            Sway_drag_force += 1 / 2. * C_drag_sway * rho * A_hitbox/14. *  u_y_zcoordinate[j] * abs(u_y_zcoordinate[j]) 
            Sway_drag_moment += 1 / 2. * C_drag_sway * rho  * A_hitbox/14. * u_y_zcoordinate[j] * abs(u_y_zcoordinate[j]) * z_dist_cog[j]

            #Acceleration of water particles
            udot_y_zcoordinate[j] = (C_06 * np.cosh(k * (z_loc[j] + d)) / C_07 * np.sin(C_01 - C_10 * y_loc[j]) - C_05 * np.cosh(2 * k * (z_loc[j] + d)) / (C_08) * np.sin(2 * (C_01 - C_10 * y_loc[j])))
            #Inertia force
            H_Force_hydro += udot_y_zcoordinate[j] * M_added_sway_plate/14.
            H_Moment_hydro += udot_y_zcoordinate[j] * M_added_sway_plate/14. * z_dist_cog[j]



#### No wave forcing, hence no variable drag force due to waves neighter inertia forces #####                                              
    else:
        ####### Horizontal forces ######
        rel_hori_vel = np.zeros(14)
        for i in range(14):
            rel_hori_vel[i] = u - xd_sway - xd_roll * z_dist_cog[i]
            # Force and moment of drag force, F = 1/2. * rho * C * v^2 * A,
            Sway_drag_force += 1/2. * C_drag_sway * rho * A_hitbox/14. * rel_hori_vel[i] * abs(rel_hori_vel[i])
            Sway_drag_moment += 1/2. * C_drag_sway * rho * A_hitbox/14. * rel_hori_vel[i] * abs(rel_hori_vel[i]) * z_dist_cog[i]
            
    # Add found values to forcing array collection
    F[0] = Sway_drag_force + C_inertia*H_Force_hydro
    F[1] = 0
    F[2] = Sway_drag_moment + C_inertia*H_Moment_hydro

    return (F)


def SwayDoublePlateForces(iteration_step, x_list, xdot_list, Wave_hlp, Current_Force, depth, Tidal_level, flow_velocity, rho, Total_time, 
                    steps, Turbine_dimensions_HWL, Wave_Force, COG_to_surface, Sway_plate_char, C_inertia, draught, characteristics_large_floater):


    # seperate data from arrays which were imported
    FFTurbine_H, FFTurbine_W, FFTurbine_L, Turbine_depth, Tur_grid_size, Turbine_Volume, N_turbines, FFTur_evalf_loc = Turbine_dimensions_HWL
    Sway_width, Sway_height, z_location_sway, C_drag_sway, M_added_sway_plate, sway_plate_num = Sway_plate_char
    Number_of_large_floaters, Width_large_floater, Length_large_floater = characteristics_large_floater
    x_sway = x_list[iteration_step, 0]
    x_heave = x_list[iteration_step, 1]
    x_roll = x_list[iteration_step, 2]
    xd_sway = xdot_list[iteration_step, 0]
#    xd_heave = xdot_list[iteration_step, 1]
    xd_roll = xdot_list[iteration_step, 2]
    u = flow_velocity
    
    A_hitbox = Sway_width * Sway_height * sway_plate_num
    M_added_sway_plate = M_added_sway_plate * sway_plate_num
    
    
    ####################3 DONT FORGET TO ADAPT THE ADDED FORMULA  ####################
    
    Sway_drag_force = 0
    Sway_drag_moment = 0
    H_Force_hydro = 0
    H_Moment_hydro = 0
    N_matrix = 3
    F = np.zeros(N_matrix)
    
#    num_grid_hori = 10
    num_grid_vert = 10


    if Current_Force == False:
        u = 0    

    if Wave_Force == True:
        t = np.linspace(0, Total_time, steps)[iteration_step]
        H, L, T = Wave_hlp

        # Fix wave direction, as negative waves are entered for negative direction
        if H < 0:
            H = - H
            Wave_direction = -1
        else:
            Wave_direction = 1
        # Wave parameters
        Wave_k = 2 * np.pi / L
        Wave_w = 2 * np.pi / T
        k = Wave_k
        w = Wave_w
        a = H / 2.
        # Depth fix, include additional water level due to tides
        d = depth + Tidal_level

        # Tricks to save time
        C_01 = w * t + u * t * k
        C_05 = 6 / 4. * (w + u * k) * w * a ** 2 * k
        C_06 = - (w + u * k) * w * a
        C_07 = np.sinh(k * d)
        C_08 = (np.cosh(k * d)) ** 3 * np.sinh(k * d)
        C_10 = Wave_direction * k

##### HORIZONTAL CONTRIBUTION #####
        if sway_plate_num == 1:
            y_coordinate = np.array([0])
        elif sway_plate_num == 2:
            y_coordinate = np.array([-Sway_height/2.,Sway_height/2.])
        elif sway_plate_num == 3:
            y_coordinate = np.array([-Sway_height, 0, Sway_height])
        elif sway_plate_num == 4:
            y_coordinate = np.array([-3 * Sway_height/2., -Sway_height/2. ,Sway_height/2., 3 * Sway_height/2.])
        elif sway_plate_num == 5:
            y_coordinate = np.array([-2 * Sway_height, -Sway_height, 0, Sway_height, 2 * Sway_height])
        


        for b in range(sway_plate_num): #loop over the floater length
            gridsize = Sway_height / num_grid_vert
            z_coordinate = np.linspace(-Turbine_depth - FFTurbine_H - Sway_height + gridsize / 2., -Turbine_depth - FFTurbine_H - gridsize / 2., num_grid_vert)
            z_loc = np.zeros(num_grid_vert)
            z_dist_cog = np.zeros(num_grid_vert)
            y_loc = np.zeros(num_grid_vert)
            for j in range(num_grid_vert):
                z_loc[j] = z_coordinate[j] + x_heave + x_roll * y_coordinate[b]
                z_dist_cog[j] = abs(z_coordinate[j]) + abs(COG_to_surface)
                y_loc[j] = x_sway + x_roll * z_dist_cog[j] + y_coordinate[b]
            
            u_y_zcoordinate = np.zeros(num_grid_vert)
            udot_y_zcoordinate = np.zeros(num_grid_vert)
            
            for j in range(num_grid_vert):
                #Velocity of water particles
                u_y_zcoordinate[j] = (Wave_w * a * np.cosh(Wave_k * (z_loc[j] + d)) / np.sinh(Wave_k * d) * np.cos(
                    Wave_w * t + u * t / L * 2 * np.pi - Wave_direction * Wave_k * y_loc[j]) + 3 / 4. * Wave_w * a ** 2 * Wave_k * np.cosh(
                    2 * Wave_k * (z_loc[j] + d)) / ((np.cosh(Wave_k * d)) ** 3 * np.sinh(Wave_k * d)) * np.cos(
                    2 * (Wave_w * t + u * t / L * 2 * np.pi - Wave_direction * Wave_k * y_loc[j]))
                    + u - xd_sway - xd_roll * z_dist_cog[j])
                #Drag force
                Sway_drag_force += 1 / 2. * C_drag_sway * rho * A_hitbox/(sway_plate_num * num_grid_vert) *  u_y_zcoordinate[j] * abs(u_y_zcoordinate[j]) 
                Sway_drag_moment += 1 / 2. * C_drag_sway * rho  * A_hitbox/(sway_plate_num * num_grid_vert) * u_y_zcoordinate[j] * abs(u_y_zcoordinate[j]) * z_dist_cog[j]
    
                #Acceleration of water particles
                udot_y_zcoordinate[j] = (C_06 * np.cosh(k * (z_loc[j] + d)) / C_07 * np.sin(C_01 - C_10 * y_loc[j]) - C_05 * np.cosh(2 * k * (z_loc[j] + d)) / (C_08) * np.sin(2 * (C_01 - C_10 * y_loc[j])))
                #Inertia force
                H_Force_hydro += udot_y_zcoordinate[j] * M_added_sway_plate/(sway_plate_num * num_grid_vert)
                H_Moment_hydro += udot_y_zcoordinate[j] * M_added_sway_plate/(sway_plate_num * num_grid_vert) * z_dist_cog[j]



#### No wave forcing, hence no variable drag force due to waves neighter inertia forces #####                                              
    else:
        ####### Horizontal forces ######
        z_coordinate = np.linspace(-Turbine_depth - FFTurbine_H - Sway_height + gridsize / 2., -Turbine_depth - FFTurbine_H - gridsize / 2., num_grid_vert)
        rel_hori_vel = np.zeros(num_grid_vert)
        for i in range(num_grid_vert):
            z_dist_cog[j] = abs(z_coordinate[j]) + abs(COG_to_surface)
            rel_hori_vel[i] = u - xd_sway - xd_roll * z_dist_cog[i]
            # Force and moment of drag force, F = 1/2. * rho * C * v^2 * A,
            Sway_drag_force += 1/2. * C_drag_sway * rho * A_hitbox/num_grid_vert * rel_hori_vel[i] * abs(rel_hori_vel[i])
            Sway_drag_moment += 1/2. * C_drag_sway * rho * A_hitbox/num_grid_vert * rel_hori_vel[i] * abs(rel_hori_vel[i]) * z_dist_cog[i]
            
    # Add found values to forcing array collection
    F[0] = Sway_drag_force + C_inertia*H_Force_hydro
    F[1] = 0
    F[2] = Sway_drag_moment + C_inertia*H_Moment_hydro

    return (F)





def SwayPlateForcesD3(iteration_step, x_list, xdot_list, Wave_hlp, Current_Force, depth, Tidal_level, flow_velocity, rho, Total_time, 
                    steps, Turbine_dimensions_HWL, Wave_Force, COG_to_surface, Sway_plate_char, C_inertia, draught, characteristics_large_floater, M_added_sway_plateD3):


    # seperate data from arrays which were imported
    FFTurbine_H, FFTurbine_W, FFTurbine_L, Turbine_depth, Tur_grid_size, Turbine_Volume, N_turbines, FFTur_evalf_loc = Turbine_dimensions_HWL
    Sway_width, Sway_height, z_location_sway, C_drag_sway, M_added_sway_plate, sway_plate_num = Sway_plate_char
    Number_of_large_floaters, Width_large_floater, Length_large_floater = characteristics_large_floater
    x_sway = x_list[iteration_step, 0]
    x_heave = x_list[iteration_step, 1]
    x_roll = x_list[iteration_step, 2]
    xd_sway = xdot_list[iteration_step, 0]
#    xd_heave = xdot_list[iteration_step, 1]
    xd_roll = xdot_list[iteration_step, 2]
    u = flow_velocity

    Sway_height = 9.
    Sway_width = Width_large_floater
    
    A_hitbox = Sway_width * Sway_height * 4 * Number_of_large_floaters #four plates and three floaters
    M_added_sway_plate = M_added_sway_plateD3
    
    
    ####################3 DONT FORGET TO ADAPT THE ADDED FORMULA  ####################
    
    Sway_drag_force = 0
    Sway_drag_moment = 0
    H_Force_hydro = 0
    H_Moment_hydro = 0
    N_matrix = 3
    F = np.zeros(N_matrix)
    
    num_grid_hori = 10
    num_grid_vert = 5


    if Current_Force == False:
        u = 0    

    if Wave_Force == True:
        t = np.linspace(0, Total_time, steps)[iteration_step]
        H, L, T = Wave_hlp

        # Fix wave direction, as negative waves are entered for negative direction
        if H < 0:
            H = - H
            Wave_direction = -1
        else:
            Wave_direction = 1
        # Wave parameters
        Wave_k = 2 * np.pi / L
        Wave_w = 2 * np.pi / T
        k = Wave_k
        w = Wave_w
        a = H / 2.
        # Depth fix, include additional water level due to tides
        d = depth + Tidal_level

        # Tricks to save time
        C_01 = w * t + u * t * k
        C_05 = 6 / 4. * (w + u * k) * w * a ** 2 * k
        C_06 = - (w + u * k) * w * a
        C_07 = np.sinh(k * d)
        C_08 = (np.cosh(k * d)) ** 3 * np.sinh(k * d)
        C_10 = Wave_direction * k

##### HORIZONTAL CONTRIBUTION #####
        gridsize_hori = FFTurbine_L/num_grid_hori
        y_coordinate = np.linspace(-FFTurbine_L/2. + gridsize_hori / 2., FFTurbine_L/2. - gridsize_hori/2., num_grid_hori)
        for b in range(num_grid_hori): #loop over the floater length
            gridsize = Sway_height / num_grid_vert
            z_coordinate = np.linspace(-draught - Sway_height + gridsize / 2., -draught - gridsize/2., num_grid_vert)
            z_loc = np.zeros(num_grid_vert)
            z_dist_cog = np.zeros(num_grid_vert)
            y_loc = np.zeros(num_grid_vert)
            for j in range(num_grid_vert):
                z_loc[j] = z_coordinate[j] + x_heave + x_roll * y_coordinate[b]
                z_dist_cog[j] = abs(z_coordinate[j]) + abs(COG_to_surface)
                y_loc[j] = x_sway + x_roll * z_dist_cog[j] + y_coordinate[b]
                 
            u_y_zcoordinate = np.zeros(num_grid_vert)
            udot_y_zcoordinate = np.zeros(num_grid_vert)
            
            for j in range(num_grid_vert):
                #Velocity of water particles
                u_y_zcoordinate[j] = (Wave_w * a * np.cosh(Wave_k * (z_loc[j] + d)) / np.sinh(Wave_k * d) * np.cos(
                    Wave_w * t + u * t / L * 2 * np.pi - Wave_direction * Wave_k * y_loc[j]) + 3 / 4. * Wave_w * a ** 2 * Wave_k * np.cosh(
                    2 * Wave_k * (z_loc[j] + d)) / ((np.cosh(Wave_k * d)) ** 3 * np.sinh(Wave_k * d)) * np.cos(
                    2 * (Wave_w * t + u * t / L * 2 * np.pi - Wave_direction * Wave_k * y_loc[j]))
                    + u - xd_sway - xd_roll * z_dist_cog[j])
                #Drag force
                Sway_drag_force += 1 / 2. * C_drag_sway * rho * A_hitbox/(num_grid_hori * num_grid_vert) *  u_y_zcoordinate[j] * abs(u_y_zcoordinate[j]) 
                Sway_drag_moment += 1 / 2. * C_drag_sway * rho  * A_hitbox/(num_grid_hori * num_grid_vert) * u_y_zcoordinate[j] * abs(u_y_zcoordinate[j]) * z_dist_cog[j]
    
                #Acceleration of water particles
                udot_y_zcoordinate[j] = (C_06 * np.cosh(k * (z_loc[j] + d)) / C_07 * np.sin(C_01 - C_10 * y_loc[j]) - C_05 * np.cosh(2 * k * (z_loc[j] + d)) / (C_08) * np.sin(2 * (C_01 - C_10 * y_loc[j])))
                #Inertia force
                H_Force_hydro += udot_y_zcoordinate[j] * M_added_sway_plate/(num_grid_hori * num_grid_vert)
                H_Moment_hydro += udot_y_zcoordinate[j] * M_added_sway_plate/(num_grid_hori * num_grid_vert) * z_dist_cog[j]



#### No wave forcing, hence no variable drag force due to waves neighter inertia forces #####                                              
    else:
        print('THE FORCING FOR THE SWAY PLATE DESIGN 3 IS NOT GIVEN FOR A SITUATION WITHOUT WAVES')
            
    # Add found values to forcing array collection
    F[0] = Sway_drag_force + C_inertia*H_Force_hydro
    F[1] = 0
    F[2] = Sway_drag_moment + C_inertia*H_Moment_hydro

    return (F)





def HeavePlateForces(iteration_step, x_list, xdot_list, Wave_hlp, Current_Force, depth, Tidal_level, flow_velocity, rho, Total_time, 
                    steps, Turbine_dimensions_HWL, Wave_Force, COG_to_surface, Heave_plate_char, C_inertia, characteristics_large_floater):


    

    # seperate data from arrays which were imported
    FFTurbine_H, FFTurbine_W, FFTurbine_L, Turbine_depth, Tur_grid_size, Turbine_Volume, N_turbines, FFTur_evalf_loc = Turbine_dimensions_HWL
    Heave_dist_x,Heave_dist_y,z_location_heave, heave_plate_bottom_dist, C_drag_sway,M_added_heave_plate, M_added_heave_plate_roll, M_added_heave_plate_2, M_added_heave_plate_roll_2, second_level_heave, height_second_level, n_plates = Heave_plate_char
    Number_of_large_floaters, Width_large_floater, Length_large_floater = characteristics_large_floater
    z_location_heave_2 = z_location_heave + abs(Heave_dist_x)/2. + abs(Width_large_floater)/2.
    x_sway = x_list[iteration_step, 0]
    x_heave = x_list[iteration_step, 1]
    x_roll = x_list[iteration_step, 2]
    xd_heave = xdot_list[iteration_step, 1]
    xd_roll = xdot_list[iteration_step, 2]
    u = flow_velocity
    

    A_hitbox1 = n_plates * Heave_dist_x * Heave_dist_y
    A_hitbox2 = n_plates * FFTurbine_L * Width_large_floater
    
    Heave_drag_force = 0
    Heave_drag_moment = 0
    V_Force_hydro = 0
    V_Moment_hydro = 0
    Heave_drag_force_2 = 0
    Heave_drag_moment_2 = 0
    V_Force_hydro_2 = 0
    V_Moment_hydro_2 = 0
    N_matrix = 3
    F = np.zeros(N_matrix)

    gridsize = Heave_dist_y / 14.
    gridsize_2 = FFTurbine_L / 14.
    y_coordinate = np.linspace(-Heave_dist_y/2+gridsize/2., Heave_dist_y/2-gridsize/2., 14)
    y_coordinate_2 = np.linspace(-FFTurbine_L/2+gridsize_2/2., FFTurbine_L/2-gridsize_2/2., 14)
    
    if Current_Force == False:
        u = 0    

    if Wave_Force == True:
        t = np.linspace(0, Total_time, steps)[iteration_step]
        H, L, T = Wave_hlp

        # Fix wave direction, as negative waves are entered for negative direction
        if H < 0:
            H = - H
            Wave_direction = -1
        else:
            Wave_direction = 1
        # Wave parameters
        Wave_k = 2 * np.pi / L
        Wave_w = 2 * np.pi / T
        k = Wave_k
        w = Wave_w
        a = H / 2.
        # Depth fix, include additional water level due to tides
        d = depth + Tidal_level

        # Tricks to save time
        C_01 = w * t + u * t * k
        C_05 = 6 / 4. * (w + u * k) * w * a ** 2 * k
        C_06 = - (w + u * k) * w * a
        C_07 = np.sinh(k * d)
        C_08 = (np.cosh(k * d)) ** 3 * np.sinh(k * d)
        C_10 = Wave_direction * k

        
        
        y_loc = np.zeros(14)
        z_loc = np.zeros(14)
        y_loc_2 = np.zeros(14)
        z_loc_2 = np.zeros(14)
        for j in range(len(z_loc)):
            y_loc[j] = y_coordinate[j] + x_sway + abs(z_location_heave) * x_roll
            z_loc[j] = z_location_heave + x_heave + y_coordinate[j] * x_roll
            y_loc_2[j] = y_coordinate_2[j] + x_sway + abs(z_location_heave_2) * x_roll
            z_loc_2[j] = z_location_heave_2 + x_heave + y_coordinate_2[j] * x_roll
        

        u_z_ycoordinate = np.zeros(14)
        udot_z_ycoordinate = np.zeros(14)
        u_z_ycoordinate_2 = np.zeros(14)
        udot_z_ycoordinate_2 = np.zeros(14)
        
        for j in range(14):
            #Velocity of water particles
            u_z_ycoordinate[j] = (w * a * np.sinh(k * (z_loc[j] + d)) / np.sinh(k * d) * np.sin(w * t + u * t / L * 2 * np.pi - Wave_direction * k * y_loc[j])
                                  + 3 / 4. * w * a ** 2 * k * np.sinh(2 * k * (z_loc[j] + d)) / ((np.cosh(k * d)) ** 3 * np.sinh(k * d))
                                  * np.sin(2 * (w * t + u * t / L * 2 * np.pi - Wave_direction * k * y_loc[j])) - xd_heave - xd_roll * y_coordinate[j])
            u_z_ycoordinate_2[j] = (w * a * np.sinh(k * (z_loc_2[j] + d)) / np.sinh(k * d) * np.sin(w * t + u * t / L * 2 * np.pi - Wave_direction * k * y_loc_2[j])
                                  + 3 / 4. * w * a ** 2 * k * np.sinh(2 * k * (z_loc_2[j] + d)) / ((np.cosh(k * d)) ** 3 * np.sinh(k * d))
                                  * np.sin(2 * (w * t + u * t / L * 2 * np.pi - Wave_direction * k * y_loc_2[j])) - xd_heave - xd_roll * y_coordinate_2[j])
            #Drag force
            Heave_drag_force += 1 / 2. * C_drag_sway * rho * A_hitbox1/14. *  u_z_ycoordinate[j] * abs(u_z_ycoordinate[j])
            Heave_drag_moment += 1 / 2. * C_drag_sway * rho * A_hitbox1/14. *  u_z_ycoordinate[j] * abs(u_z_ycoordinate[j]) * y_coordinate[j]
            Heave_drag_force_2 += 1 / 2. * C_drag_sway * rho * A_hitbox2/14. *  u_z_ycoordinate_2[j] * abs(u_z_ycoordinate_2[j])
            Heave_drag_moment_2 += 1 / 2. * C_drag_sway * rho * A_hitbox2/14. *  u_z_ycoordinate_2[j] * abs(u_z_ycoordinate_2[j]) * y_coordinate_2[j]
            
            #Acceleration of water particles
            udot_z_ycoordinate[j] = C_06 * np.sinh(k * (z_loc[j] + d)) / C_07 * np.cos(C_01 - C_10 * y_loc[j]) - C_05 * np.sinh(2 * k * (z_loc[j] + d)) / (C_08) * np.cos(2 * (C_01 - C_10 * y_loc[j]))
            udot_z_ycoordinate_2[j] = C_06 * np.sinh(k * (z_loc_2[j] + d)) / C_07 * np.cos(C_01 - C_10 * y_loc_2[j]) - C_05 * np.sinh(2 * k * (z_loc_2[j] + d)) / (C_08) * np.cos(2 * (C_01 - C_10 * y_loc_2[j]))
            #Inertia force
            V_Force_hydro += udot_z_ycoordinate[j] * M_added_heave_plate/14.
            V_Moment_hydro += udot_z_ycoordinate[j] * M_added_heave_plate/14. * y_coordinate[j]
            V_Force_hydro_2 += udot_z_ycoordinate_2[j] * M_added_heave_plate_2/14.
            V_Moment_hydro_2 += udot_z_ycoordinate_2[j] * M_added_heave_plate_2/14. * y_coordinate_2[j]

#### No wave forcing, hence no variable drag force due to waves neighter inertia forces #####                                              
    else:
        ####### Horizontal forces ######
        rel_vert_vel = np.zeros(14)
        rel_vert_vel_2 = np.zeros(14)
        for i in range(14):
            rel_vert_vel[i] = - xd_heave - xd_roll * y_coordinate[i]
            rel_vert_vel_2[i] = - xd_heave - xd_roll * y_coordinate_2[i]
            # Force and moment of drag force, F = 1/2. * rho * C * v^2 * A,
            Heave_drag_force += 1 / 2. * C_drag_sway * rho * A_hitbox1/14. *  rel_vert_vel[i] * abs(rel_vert_vel[i])
            Heave_drag_moment += 1 / 2. * C_drag_sway * rho * A_hitbox1/14. *  rel_vert_vel[i] * abs(rel_vert_vel[i]) * y_coordinate[j]
            Heave_drag_force_2 += 1 / 2. * C_drag_sway * rho * A_hitbox2/14. *  rel_vert_vel_2[i] * abs(rel_vert_vel_2[i])
            Heave_drag_moment_2 += 1 / 2. * C_drag_sway * rho * A_hitbox2/14. *  rel_vert_vel_2[i] * abs(rel_vert_vel_2[i]) * y_coordinate_2[j]
            
#    Heave_drag_force = 0  #this is for the upper deck only
#    Heave_drag_moment = 0  #this is for the upper deck only
#    V_Force_hydro = 0  #this is for the upper deck only
#    V_Moment_hydro = 0  #this is for the upper deck only
    
    # Add found values to forcing array collection
    F[0] = 0
    F[1] = Heave_drag_force + C_inertia * V_Force_hydro
    F[2] = Heave_drag_moment + C_inertia * V_Moment_hydro
    
    if second_level_heave == True:
        F[0] += 0
        F[1] += Heave_drag_force_2 + C_inertia * V_Force_hydro_2
        F[2] += Heave_drag_moment_2 + C_inertia * V_Moment_hydro_2

    return (F)


def SloshDamper(iteration_step, x_list, xddot_list, rho, Total_time, steps, slosh_kin_list, Slosh_char, g, z_location_bottom_floater,Slosh_Force_test):
    
    x_y0 = slosh_kin_list[iteration_step-1, 0] #y-location at beginning of time step
    xdot_S0 = slosh_kin_list[iteration_step-1, 2]
    
    x_roll = x_list[iteration_step-1, 2]
    xdd_sway = xddot_list[iteration_step-1, 0]
    xdd_heave = xddot_list[iteration_step-1, 1]
    xdd_roll = xddot_list[iteration_step-1, 2]
    F = np.zeros(3)
    E_matrix = np.zeros((3,3))
    
    
    
    if iteration_step ==0:
        if Slosh_Force_test==True:
            F[0] = 0
            F[1] = 0
            F[2] = 0
            slosh_kin_list[iteration_step, 0] = 1
            slosh_kin_list[iteration_step, 1] = 0
            slosh_kin_list[iteration_step, 2] = 0
            slosh_kin_list[iteration_step, 3] = 0
            slosh_kin_list[iteration_step, 4] = 0
            slosh_kin_list[iteration_step, 5] = 0
        else:
            F[0] = 0
            F[1] = 0
            F[2] = 0
            slosh_kin_list[iteration_step, 0] = 0
            slosh_kin_list[iteration_step, 1] = 0
            slosh_kin_list[iteration_step, 2] = 0
            slosh_kin_list[iteration_step, 3] = 0
            slosh_kin_list[iteration_step, 4] = 0
            slosh_kin_list[iteration_step, 5] = 0
    else:
        Slosh_length, Slosh_water_level, Slosh_mass = Slosh_char
        t = np.linspace(0, Total_time, steps)
        dt = t[2]-t[1]
        
        b = Slosh_length
        h = 2 * Slosh_water_level
        m = Slosh_mass
        Cd = 1.9
        if Slosh_Force_test==True:
            total_cylinder_width = 0#m
#            total_cylinder_width = 15#m
        else:
            total_cylinder_width = 15#m
            # total_cylinder_width = 30#m
        A = total_cylinder_width * Slosh_water_level
         
        

        
        
        if Slosh_Force_test==True:
            x_roll =0
            xdd_sway =0
            xdd_heave =0
            xdd_roll =0
        
        dz_dy0 = 0 
        x_z0 = 0
        if x_y0 >= -b/6. and x_y0<= b/6.:
            dz_dy0 = 6 * h * x_y0 / b**2
            x_z0 = 3 * h * x_y0**2 / b**2
        elif x_y0>b/6.:
            dz_dy0 = b*h/(9*(x_y0-b/2.)**2)
            x_z0 = - h/4. - b * h / (9*(x_y0-b/2.))
        else:
            dz_dy0 = -b*h/(9*(abs(x_y0)-b/2.)**2)
            x_z0 = - h/4. - b * h / (9*(abs(x_y0)-b/2.))
        
            
        theta0 = np.arctan(dz_dy0) + np.arctan(x_roll)
        
        mass_y_coordinate_to_COG = x_y0
        mass_z_coordinate_to_COG = z_location_bottom_floater + Slosh_water_level/2. + x_z0 #this is a negative value in general, because it is below the COG
        
        a_inertia = np.sqrt((-g - xdd_heave - mass_y_coordinate_to_COG * xdd_roll)**2+(-xdd_sway + mass_z_coordinate_to_COG * xdd_roll)**2)
        phi_a_inertia = np.arctan((-xdd_sway + mass_z_coordinate_to_COG * xdd_roll)/(-g - xdd_heave - mass_y_coordinate_to_COG * xdd_roll))
        
        F_normal = np.cos(theta0 + phi_a_inertia) * a_inertia * m
        F_normal_sway = F_normal * np.sin(theta0)
        F_normal_heave = - F_normal * np.cos(theta0)
        F[0] = F_normal_sway
        # F[1] = F_normal_heave + m*g #The mass was already added in the draught of the structure
        F[1] = F_normal_heave #The mass was already added in the draught of the structure
        F[2] = F_normal_sway * -mass_z_coordinate_to_COG + F_normal_heave * mass_y_coordinate_to_COG
        
        steppers = 5
        dt_new = dt/float(steppers)
        
        x_y = np.zeros(steppers+1)
        xdot_S = np.zeros(steppers+1)
        
        x_y[0] = x_y0
        xdot_S[0] = xdot_S0
        
        for i in range(steppers):
            dz_dy0 = 0 
            if x_y[i] >= -b/6. and x_y[i]<= b/6.:
                dz_dy0 = 6 * h * x_y[i] / b**2
            elif x_y[i]>b/6.:
                dz_dy0 = b*h/(9*(x_y[i]-b/2.)**2)
            else:
                dz_dy0 = -b*h/(9*(abs(x_y[i])-b/2.)**2)   
            theta0 = np.arctan(dz_dy0) + np.arctan(x_roll)
            
            a_res0 = -(np.sin(theta0 + phi_a_inertia) * a_inertia) - rho * Cd * A * xdot_S[i] * abs(xdot_S[i])/(2*m)
            dS_old = xdot_S[i] * dt_new + 0.5 * a_res0 * dt_new**2
            if x_y[i] >= -b/6. and x_y[i]<= b/6.:
                dy_old = dS_old / np.sqrt(1+(6*h*abs(x_y[i])/b**2)**2)
            else:
                dy_old = dS_old / np.sqrt(1+ (b*h/(9*(abs(x_y[i])-b/2.)**2))**2)
            x_y[i+1] = x_y[i] + dy_old
            
            dz_dy1 = 0 
            if x_y[i+1] >= -b/6. and x_y[i+1]<= b/6.:
                dz_dy1 = 6 * h * x_y[i+1] / b**2
            elif x_y[i+1]>b/6.:
                dz_dy1 = b*h/(9*(x_y[i+1]-b/2.)**2)
            else:
                dz_dy1 = -b*h/(9*(abs(x_y[i+1])-b/2.)**2)
            theta1 = np.arctan(dz_dy1) + np.arctan(x_roll)
            
            a_res1 = -(np.sin(theta1 + phi_a_inertia) * a_inertia) - rho * Cd * A * xdot_S[i] * abs(xdot_S[i])/(2*m)
            xdot_S[i+1] = xdot_S[i] + (a_res0 + a_res1)/2. * dt_new
            dS_new = xdot_S[i] * dt_new + 0.5 * (a_res0 + a_res1)/2. * dt_new**2
            
            if (x_y[i]+x_y[i+1])/2. >= -b/6. and (x_y[i]+x_y[i+1])/2.<= b/6.:
                dy_new = dS_new / np.sqrt(1+(6*h*abs((x_y[i]+x_y[i+1])/2.)/b**2)**2)
                xdot_y_last = xdot_S[i+1] / np.sqrt(1+(6*h*abs((x_y[i]+x_y[i+1])/2.)/b**2)**2)
            else:
                dy_new = dS_new / np.sqrt(1+ (b*h/(9*(abs((x_y[i]+x_y[i+1])/2.)-b/2.)**2))**2)
                xdot_y_last = xdot_S[i+1] / np.sqrt(1+ (b*h/(9*(abs((x_y[i]+x_y[i+1])/2.)-b/2.)**2))**2)
            
            x_y[i+1] = x_y[i] + dy_new

        if x_y[steppers] >= -b/6. and x_y[steppers]<= b/6.:
            x_z1 = 3 * h * x_y[steppers]**2 / b**2
        elif x_y[steppers]>b/6.:
            x_z1 = - h/4. - b * h / (9*(x_y[steppers]-b/2.))
        else:
            x_z1 = - h/4. - b * h / (9*(abs(x_y[steppers])-b/2.))


        slosh_kin_list[iteration_step, 0] = x_y[steppers]
        slosh_kin_list[iteration_step, 1] = xdot_y_last
        slosh_kin_list[iteration_step, 2] = xdot_S[steppers]
        slosh_kin_list[iteration_step, 3] = x_z1
        slosh_kin_list[iteration_step, 4] = (a_res0+a_res1)/2.
        slosh_kin_list[iteration_step, 5] = dz_dy0

#    if iteration_step > 4 :
#        E_matrix[0,0] = abs(F[0]/xdd_sway)
#        E_matrix[0,1]= abs(F[0]/xdd_heave)
#        E_matrix[0,2]= abs(F[0]/xdd_roll)
#        E_matrix[1,0]= abs(F[1]/xdd_sway)
#        E_matrix[1,1]= abs(F[1]/xdd_heave)
#        E_matrix[1,2]= abs(F[1]/xdd_roll)
#        E_matrix[2,0]= abs(F[2]/xdd_sway)
#        E_matrix[2,1]= abs(F[2]/xdd_heave)
#        E_matrix[2,2]= abs(F[2]/xdd_roll)
        
#        E_matrix[0,0]= (F[0]/xdd_sway)
#        E_matrix[0,1]= (F[0]/xdd_heave)
#        E_matrix[0,2]= (F[0]/xdd_roll)
#        E_matrix[1,0]= (F[1]/xdd_sway)
#        E_matrix[1,1]= (F[1]/xdd_heave)
#        E_matrix[1,2]= (F[1]/xdd_roll)
#        E_matrix[2,0]= (F[2]/xdd_sway)
#        E_matrix[2,1]= (F[2]/xdd_heave)
#        E_matrix[2,2]= (F[2]/xdd_roll)
        
#        E_matrix[0,0]= (F[0]/xdd_sway)
#        E_matrix[1,0]= (F[0]/xdd_heave)
#        E_matrix[2,0]= (F[0]/xdd_roll)
#        E_matrix[0,1]= (F[1]/xdd_sway)
#        E_matrix[1,1]= (F[1]/xdd_heave)
#        E_matrix[2,1]= (F[1]/xdd_roll)
#        E_matrix[0,2]= (F[2]/xdd_sway)
#        E_matrix[1,2]= (F[2]/xdd_heave)
#        E_matrix[2,2]= (F[2]/xdd_roll)

    # print(F[0],F[1],F[2],xdd_sway,xdd_heave,xdd_roll)

    return (F, slosh_kin_list, E_matrix)
